/*
 * File   :     AnnoArea.java
 * Author :     Tis Veugen
 * Date   :     1999-11-24
 * PBN    :     2.0
 *
 * History
 * -------
 * 1999-11-15 Check area width
 * 1999-11-24 Ref to CallNote becomes CardNote
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class AnnoArea
{
  public final char             CHAR_BACKSLASH = '\\';

  private Component             mObserver;
  private int                   mAreaWidth;
  private PbnGameData           mGameData;
  private PbnGameTags           mGameTags;
  private PbnPlayAdmin          mPlayAdmin;
  private Image                 mImage;
  private Graphics              mGraphics;
  private int                   mNrCalls;
  private int                   mNrCards;
  private AnnoRange             mLinesIdent;
  private AnnoRange             mLinesAuction;
  private AnnoRange             mLinesPlay;
  private AnnoRange []          mLinesCalls;
  private AnnoRange []          mLinesCards;
  private AnnoRange             mLinesSuppl;

  private Font                  mFont = new Font( "Monospaced", Font.PLAIN, 14 );
  private FontMetrics           mFontMetrics;
  private int                   mCharWidth;
  private int                   mCharHeigth;
  private int                   mMaxWidth;
  private int                   mAnnoY;
  private int                   mYImage;
  private AnnoRange             mLinesMax;

  private static String []      maNagTexts =
  {
    "no annotation",
    "good call",
    "poor call",
    "very good call",
    "very poor call",
    "speculative call",
    "questionable call",
    "good card",
    "poor card",
    "very good card",
    "very poor card",
    "speculative card",
    "questionable card"
  };

  public                        AnnoArea(
  Component                     oComponent,
  int                           iAreaWidth )
  {
    mObserver  = oComponent;
    if ( iAreaWidth < 256 )
    {
      iAreaWidth = 256;
    }
    mAreaWidth = iAreaWidth;
  }

  public Image                  ComputeImage(
  PbnGameData                   oGameData,
  PbnGameTags                   oGameTags,
  PbnPlayAdmin                  oPlayAdmin )
  {
    mGameData  = oGameData;
    mGameTags  = oGameTags;
    mPlayAdmin = oPlayAdmin;

    Toolkit                     tk = Toolkit.getDefaultToolkit();
    mFontMetrics = tk.getFontMetrics( mFont );
    mCharWidth  = mFontMetrics.charWidth( 'A' );
    mCharHeigth = mFontMetrics.getHeight();
    mYImage = mCharHeigth - 12; // 12=Height of image

    mLinesMax = new AnnoRange();
    mMaxWidth = mAreaWidth;
    mAnnoY = 2;
    WidthAnnotation();
    mAnnoY += 2 + 8;

    mLinesIdent   = new AnnoRange();
    mLinesAuction = new AnnoRange();
    mLinesPlay    = new AnnoRange();
    mLinesSuppl   = new AnnoRange();

    mLinesCalls = new AnnoRange[ mNrCalls ];
    for ( int i = 0; i < mNrCalls; i++ )
    {
      mLinesCalls[ i ] = new AnnoRange();
    }

    mLinesCards = new AnnoRange[ mNrCards ];
    for ( int i = 0; i < mNrCards; i++ )
    {
      mLinesCards[ i ] = new AnnoRange();
    }

    mImage = mObserver.createImage( mMaxWidth+2, mAnnoY );
    mGraphics = mImage.getGraphics();

    try
    {
      mAnnoY = 2;
      mGraphics.setFont( mFont );
      DrawAnnotation();
    }
    finally
    {
      mGraphics.dispose();
    }

    return mImage;
  }

  private void                  SetMaxWidth(
  int                           iWidth )
  {
    if ( mMaxWidth < iWidth )
    {
      mMaxWidth = iWidth;
    }
  }

  private void                  NextLine()
  {
    mAnnoY += mCharHeigth;
  }

  private void                  DrawAnnotation()
  {
    PbnMoveAnno                 lMoveAnno;

    mGraphics.setColor( Color.black );

    mLinesIdent.Begin = mAnnoY;
    DrawSection( PbnGen.SECTION_IDENT );
    mLinesIdent.End = mAnnoY;

    mLinesAuction.Begin = mAnnoY;
    DrawSection( PbnGen.SECTION_AUCTION );
    mLinesAuction.End = mAnnoY;

    PbnAuction          lAuction = mGameData.GetAuction();
    for ( int iCall = 0; iCall < mNrCalls; iCall++ )
    {
      mLinesCalls[ iCall ].Begin = mAnnoY;
      lMoveAnno = mGameTags.GetCallAnno( iCall );
      DrawMoveAnno( lMoveAnno, lAuction.GetCall( iCall ), null );
      mLinesCalls[ iCall ].End = mAnnoY;
    }

    mLinesPlay.Begin = mAnnoY;
    DrawSection( PbnGen.SECTION_PLAY );
    mLinesPlay.End = mAnnoY;

    PbnPlayCard         lPlayCard;
    for ( int iCard = 0; iCard < mNrCards; iCard++ )
    {
      mLinesCards[ iCard ].Begin = mAnnoY;
      lPlayCard = mPlayAdmin.GetPlayCard( iCard );
      lMoveAnno = mGameTags.GetCardAnno( iCard / PbnSide.NUMBER
                                       , lPlayCard.GetSide() );
      DrawMoveAnno( lMoveAnno, null, lPlayCard.GetCard() );
      mLinesCards[ iCard ].End = mAnnoY;
    }

    mLinesSuppl.Begin = mAnnoY;
    DrawSection( PbnGen.SECTION_SUPPL );
    mLinesSuppl.End = mAnnoY;
  }

  private int                   DrawCall(
  int                           iX,
  PbnCall                       oCall )
  {
    int                         w = 0;
    String                      lString;

    switch ( oCall.GetType() )
    {
    case PbnCall.TYPE_PASS:
    case PbnCall.TYPE_DBL:
    case PbnCall.TYPE_RDB:
      lString = oCall.toString();
      mGraphics.drawString( lString, iX, mAnnoY + mCharHeigth );
      w = mFontMetrics.stringWidth( lString ) + 2;
      break;

    case PbnCall.TYPE_REAL:
    case PbnCall.TYPE_INSUFF:
      lString = "" + oCall.GetNrTricks();
      mGraphics.drawString( lString, iX, mAnnoY + mCharHeigth );
      w = mFontMetrics.stringWidth( lString ) + 2;

      Image mTrumpImage = MyImages.Get12( oCall.GetTrump() );
      mGraphics.drawImage( mTrumpImage, iX + w, mAnnoY + mYImage, mObserver );
      w += mTrumpImage.getWidth( mObserver );
      break;

    default:
      break;
    }

    return w;
  }

  private int                   DrawCard(
  int                           iX,
  PbnCard                       oCard )
  {
    int                         w = 0;
    String                      lString;
    PbnSuit                     lSuit = oCard.GetSuit();
    PbnRank                     lRank = oCard.GetRank();

    if ( ! ( lSuit.IsValid()
          && lRank.IsNormal() ) )
    {
      lString = "-";
      mGraphics.drawString( lString, iX, mAnnoY + mCharHeigth );
      w = mFontMetrics.stringWidth( lString );
    }
    else
    {
      Image mSuitImage = MyImages.Get12( lSuit );
      mGraphics.drawImage( mSuitImage, iX, mAnnoY + mYImage, mObserver );
      w = mSuitImage.getWidth( mObserver ) + 2;

      lString = lRank.toCharacter();
      mGraphics.drawString( lString, iX + w, mAnnoY + mCharHeigth );
      w += mFontMetrics.stringWidth( lString );
    }

    return w;
  }

  private void                  DrawComment(
  PbnComment                    oComment )
  {
    String []                   laStrings = oComment.GetStrings();
    int                         n = laStrings.length;

    for ( int i = 0; i < n; i++ )
    {
      DrawCommentLine( laStrings[ i ] );
    }
  }

  private void                  DrawCommentLine(
  String                        oString )
  {
    int                         n = oString.length();
    int                         iWidth = 0;
    int                         iNrChars  = 0;
    int                         iNrImages = 0;
    boolean                     bBackslash = false;
    PbnSuit                     lSuit = new PbnSuit();
    char                        c;
    int                         x = 2;
    int                         j = -1;

    switch ( oString.charAt( 0 ) )
    {
    case PbnComment.BEGIN_END:
      oString = oString.substring( 2, n-1 );
      break;
    case PbnComment.BEGIN:
    case PbnComment.EOL:
      oString = oString.substring( 2 );
      break;
    case PbnComment.END:
      oString = oString.substring( 1, n-1 );
      break;
    case PbnComment.NEXT:
      oString = oString.substring( 1 );
      break;
    }

    n = oString.length();
    for ( int i = 0; i < n; i++ )
    {
      c = oString.charAt( i );
      if ( bBackslash )
      {
        bBackslash = false;
        switch ( c )
        {
        case 'c':
        case 'C':
          lSuit.Set( PbnSuit.CLUBS );
          break;
        case 'd':
        case 'D':
          lSuit.Set( PbnSuit.DIAMONDS );
          break;
        case 'h':
        case 'H':
          lSuit.Set( PbnSuit.HEARTS );
          break;
        case 's':
        case 'S':
          lSuit.Set( PbnSuit.SPADES );
          break;
        }
        if ( lSuit.IsValid() )
        {
          if ( j >= 0 )
          {
            String      lSubstring = oString.substring( j, i-1 );
            mGraphics.drawString( lSubstring, x, mAnnoY + mCharHeigth );
            x += mFontMetrics.stringWidth( lSubstring );
            j = -1;
          }

          Image lSuitImage = MyImages.Get12( lSuit );
          mGraphics.drawImage( lSuitImage, x, mAnnoY + mYImage, mObserver );
          x += lSuitImage.getWidth( mObserver );
          lSuit.Set( PbnSuit.NONE );
        }
        else
        { /*
           * Count '\' and 'c' at next loop.
           */
          if ( j == -1 ) j = i-1;
        }
      }
      else
      if ( c == CHAR_BACKSLASH )
      {
        bBackslash = true;
      }
      else
      {
        if ( j == -1 ) j = i;
      }
    }

    if ( bBackslash )
    { /*
       * String ends with '\'.
       */
      if ( j == -1 ) j = n-1;
    }

    if ( j >= 0 )
    {
      String            lSubstring = oString.substring( j );
      mGraphics.drawString( lSubstring, x, mAnnoY + mCharHeigth );
    }

    NextLine();
  }

  private void                  DrawHeader(
  String                        oString )
  {
    oString = "  " + oString + "  ";

    int                         w = mFontMetrics.stringWidth( oString );
    int                         x = 2 + mAreaWidth/2 - w/2;
    int                         y = mAnnoY + mCharHeigth - 6;

    mGraphics.drawLine( 0, y, x-4, y );
    mGraphics.drawLine( x+w+4, y, mAreaWidth-1, y );

    mGraphics.setColor( PanelScreen.ColorPink );
    mGraphics.drawString( oString, x, mAnnoY + mCharHeigth );
    mGraphics.setColor( Color.black );

    NextLine();
  }

  private void                  DrawMoveAnno(
  PbnMoveAnno                   oMoveAnno,
  PbnCall                       oCall,
  PbnCard                       oCard )
  {
    PbnComment                  lComment;
    PbnNote                     lNote;
    PbnSuffix                   lSuffix;
    PbnNag []                   laNags;
    PbnMoveNote                 lMoveNote;
    int                         x;

    lComment = oMoveAnno.GetComment( PbnCommentAdmin.TYPE_MOVE );
    DrawComment( lComment );

    lNote = oMoveAnno.GetNote();
    if ( lNote.IsValid() )
    {
      x = 2;
      if ( oCall != null )
      {
        x += DrawCall( x, oCall );
        lMoveNote = mGameTags.GetCallNote( lNote.Get() );
        lComment = mGameTags.GetCallNoteComment( lNote.Get() );
      }
      else
      {
        x += DrawCard( x, oCard );
        lMoveNote = mGameTags.GetCardNote( lNote.Get() );
        lComment = mGameTags.GetCardNoteComment( lNote.Get() );
      }
      x += DrawNoteRef( x, lNote );
      DrawNoteText( x, lMoveNote );

      DrawComment( lComment );

      lComment = oMoveAnno.GetComment( PbnCommentAdmin.TYPE_NOTE_REF );
      DrawComment( lComment );
    }

    lSuffix = oMoveAnno.GetSuffix();
    if ( lSuffix.IsValid() )
    {
      x = 2;
      if ( oCall != null )
      {
        x += DrawCall( x, oCall );
        DrawSuffix( x, lSuffix, PbnGen.SECTION_AUCTION );
      }
      else
      {
        x += DrawCard( x, oCard );
        DrawSuffix( x, lSuffix, PbnGen.SECTION_PLAY );
      }

      lComment = oMoveAnno.GetComment( PbnCommentAdmin.TYPE_SUFFIX );
      DrawComment( lComment );
    }

    laNags = oMoveAnno.GetNags();
    int iNrNags = laNags.length;
    if ( iNrNags > 0 )
    {
      x = 2;
      if ( oCall != null )
      {
        x += DrawCall( x, oCall );
      }
      else
      {
        x += DrawCard( x, oCard );
      }

      for ( int iNag = 0; iNag < iNrNags; iNag++ )
      {
        x += DrawNag( x, laNags[ iNag ] );
      }
      NextLine();

      lComment = oMoveAnno.GetComment( PbnCommentAdmin.TYPE_NAG );
      DrawComment( lComment );
    }
  }

  private int                   DrawNag(
  int                           iX,
  PbnNag                        oNag )
  {
    String                      lString = " $" + oNag.Get();

    mGraphics.drawString( lString, iX, mAnnoY + mCharHeigth );

    return mFontMetrics.stringWidth( lString );
  }

  private int                   DrawNoteRef(
  int                           iX,
  PbnNote                       oNote )
  {
    String                      lString = " (" + oNote.Get() + ")";

    mGraphics.setColor( PanelScreen.ColorDarkGreen );
    mGraphics.drawString( lString, iX, mAnnoY + mCharHeigth );
    mGraphics.setColor( Color.black );

    return mFontMetrics.stringWidth( lString );
  }

  private void                  DrawNoteText(
  int                           iX,
  PbnMoveNote                   oMoveNote )
  {
    mGraphics.drawString( oMoveNote.GetTagValue(), iX, mAnnoY + mCharHeigth );

    NextLine();
  }

  private void                  DrawSectionTags(
  Enumeration                   oeTags )
  {
    PbnTagId                    lTagId;
    PbnComment                  lComment;

    while ( oeTags.hasMoreElements()  )
    {
      lTagId = (PbnTagId) oeTags.nextElement();
      lComment = mGameTags.GetTagComment( lTagId );
      DrawComment( lComment );
    }
  }

  private void                  DrawSection(
  int                           iSection )
  {
    PbnComment                  lComment;
    PbnTagId                    lTagId = new PbnTagId();

    switch ( iSection )
    {
    case PbnGen.SECTION_IDENT:
      lComment = mGameTags.GetGameComment();
      DrawComment( lComment );

      DrawSectionTags( PbnTagId.GetMtsTags() );
      DrawSectionTags( PbnTagId.GetOptionalTags() );
      break;
    case PbnGen.SECTION_AUCTION:
      DrawHeader( "AUCTION" );
      lTagId.Set( PbnTagId.AUCTION );
      lComment = mGameTags.GetTagComment( lTagId );
      DrawComment( lComment );
      break;
    case PbnGen.SECTION_PLAY:
      DrawHeader( "PLAY" );
      lTagId.Set( PbnTagId.PLAY );
      lComment = mGameTags.GetTagComment( lTagId );
      DrawComment( lComment );
      break;
    case PbnGen.SECTION_SUPPL:
      DrawSectionTags( PbnTagId.GetSupplementalTags() );
      break;
    }
  }

  private void                  DrawSuffix(
  int                           iX,
  PbnSuffix                     oSuffix,
  int                           iSection )
  {
    int                         iIndex = oSuffix.Get();

    if ( iSection != PbnGen.SECTION_AUCTION )
    {
      iIndex += PbnSuffix.NUMBER-1;
    }

    mGraphics.drawString( " " + maNagTexts[ iIndex ], iX, mAnnoY + mCharHeigth );

    NextLine();
  }


  public void                   ShowSection(
  int                           iSection )
  {
    switch ( iSection )
    {
    case PbnGen.SECTION_IDENT:
      ShowNrLines( mLinesIdent );
      break;
    case PbnGen.SECTION_AUCTION:
      ShowNrLines( mLinesAuction );
      break;
    case PbnGen.SECTION_PLAY:
      ShowNrLines( mLinesPlay );
      break;
    case PbnGen.SECTION_SUPPL:
      ShowNrLines( mLinesSuppl );
      break;
    }
  }

  public void                   ShowCall(
  int                           iCall )
  {
    ShowNrLines( mLinesCalls[ iCall ] );
  }

  public void                   ShowCard(
  int                           iCard )
  {
    ShowNrLines( mLinesCards[ iCard ] );
  }

  private void                  ShowNrLines(
  AnnoRange                     oAnnoRange )
  {
    mLinesMax = oAnnoRange;
  }

  public AnnoRange              GetRange()
  {
    return mLinesMax;
  }

  private void                  WidthAnnotation()
  {
    PbnMoveAnno                 lMoveAnno;

    WidthSection( PbnGen.SECTION_IDENT );

    WidthSection( PbnGen.SECTION_AUCTION );
    PbnAuction          lAuction = mGameData.GetAuction();
    mNrCalls = lAuction.GetNrCalls();
    for ( int iCall = 0; iCall < mNrCalls; iCall++ )
    {
      lMoveAnno = mGameTags.GetCallAnno( iCall );
      WidthMoveAnno( lMoveAnno, lAuction.GetCall( iCall ), null );
    }

    WidthSection( PbnGen.SECTION_PLAY );
    int                 iCard = 0;
    PbnPlayCard         lPlayCard;
    while ( ! mPlayAdmin.PlayIsDone( iCard ) )
    {
      lPlayCard = mPlayAdmin.GetPlayCard( iCard );
      lMoveAnno = mGameTags.GetCardAnno( iCard / PbnSide.NUMBER
                                       , lPlayCard.GetSide() );
      WidthMoveAnno( lMoveAnno, null, lPlayCard.GetCard() );
      iCard++;
    }
    mNrCards = iCard;
  }

  private int                   WidthCall(
  PbnCall                       oCall )
  {
    int                         w = 0;
    String                      lString;

    switch ( oCall.GetType() )
    {
    case PbnCall.TYPE_PASS:
    case PbnCall.TYPE_DBL:
    case PbnCall.TYPE_RDB:
      lString = oCall.toString();
      w = mFontMetrics.stringWidth( lString ) + 2;
      break;

    case PbnCall.TYPE_REAL:
    case PbnCall.TYPE_INSUFF:
      lString = "" + oCall.GetNrTricks();
      w = mFontMetrics.stringWidth( lString ) + 2;

      Image mTrumpImage = MyImages.Get12( oCall.GetTrump() );
      w += mTrumpImage.getWidth( mObserver );
      break;

    default:
      break;
    }

    return w;
  }

  private int                   WidthCard(
  PbnCard                       oCard )
  {
    int                         w = 0;
    String                      lString;
    PbnSuit                     lSuit = oCard.GetSuit();
    PbnRank                     lRank = oCard.GetRank();

    if ( ! ( lSuit.IsValid()
          && lRank.IsNormal() ) )
    {
      lString = "-";
      w = mFontMetrics.stringWidth( lString ) + 2;
    }
    else
    {
      Image mSuitImage = MyImages.Get12( lSuit );
      w = mSuitImage.getWidth( mObserver ) + 2;

      lString = lRank.toCharacter();
      w += mFontMetrics.stringWidth( lString );
    }

    return w;
  }

  private void                  WidthComment(
  PbnComment                    oComment )
  {
    String []                   laStrings = oComment.GetStrings();
    int                         n = laStrings.length;

    for ( int i = 0; i < n; i++ )
    {
      WidthCommentLine( laStrings[ i ] );
    }
  }

  private void                  WidthCommentLine(
  String                        oString )
  {
    int                         n = oString.length();
    int                         iWidth = 0;
    int                         iNrChars  = 0;
    int                         iNrImages = 0;
    boolean                     bBackslash = false;
    PbnSuit                     lSuit = new PbnSuit();
    char                        c;
    int                         x = 2;
    int                         j = -1;

    switch ( oString.charAt( 0 ) )
    {
    case PbnComment.BEGIN_END:
      oString = oString.substring( 2, n-1 );
      break;
    case PbnComment.BEGIN:
    case PbnComment.EOL:
      oString = oString.substring( 2 );
      break;
    case PbnComment.END:
      oString = oString.substring( 1, n-1 );
      break;
    case PbnComment.NEXT:
      oString = oString.substring( 1 );
      break;
    }

    n = oString.length();
    for ( int i = 0; i < n; i++ )
    {
      c = oString.charAt( i );
      if ( bBackslash )
      {
        bBackslash = false;
        switch ( c )
        {
        case 'c':
        case 'C':
          lSuit.Set( PbnSuit.CLUBS );
          break;
        case 'd':
        case 'D':
          lSuit.Set( PbnSuit.DIAMONDS );
          break;
        case 'h':
        case 'H':
          lSuit.Set( PbnSuit.HEARTS );
          break;
        case 's':
        case 'S':
          lSuit.Set( PbnSuit.SPADES );
          break;
        }
        if ( lSuit.IsValid() )
        {
          if ( j >= 0 )
          {
            String      lSubstring = oString.substring( j, i-1 );
            x += mFontMetrics.stringWidth( lSubstring );
            j = -1;
          }

          Image lSuitImage = MyImages.Get12( lSuit );
          x += lSuitImage.getWidth( mObserver );
          lSuit.Set( PbnSuit.NONE );
        }
        else
        { /*
           * Count '\' and 'c' at next loop.
           */
          if ( j == -1 ) j = i-1;
        }
      }
      else
      if ( c == CHAR_BACKSLASH )
      {
        bBackslash = true;
      }
      else
      {
        if ( j == -1 ) j = i;
      }
    }

    if ( bBackslash )
    { /*
       * String ends with '\'.
       */
      if ( j == -1 ) j = n-1;
    }

    if ( j >= 0 )
    {
      String            lSubstring = oString.substring( j );
      x += mFontMetrics.stringWidth( lSubstring );
    }

    NextLine();
    SetMaxWidth( x );
  }

  private void                  WidthMoveAnno(
  PbnMoveAnno                   oMoveAnno,
  PbnCall                       oCall,
  PbnCard                       oCard )
  {
    int                         iWidth = 0;
    PbnComment                  lComment;
    PbnNote                     lNote;
    PbnSuffix                   lSuffix;
    PbnNag []                   laNags;
    PbnMoveNote                 lMoveNote;
    int                         w;

    lComment = oMoveAnno.GetComment( PbnCommentAdmin.TYPE_MOVE );
    WidthComment( lComment );

    lNote = oMoveAnno.GetNote();
    if ( lNote.IsValid() )
    {
      w = 2;
      if ( oCall != null )
      {
        w += WidthCall( oCall );
        lMoveNote = mGameTags.GetCallNote( lNote.Get() );
        lComment = mGameTags.GetCallNoteComment( lNote.Get() );
      }
      else
      {
        w += WidthCard( oCard );
        lMoveNote = mGameTags.GetCardNote( lNote.Get() );
        lComment = mGameTags.GetCardNoteComment( lNote.Get() );
      }
      w += WidthNoteRef( lNote );
      w += WidthNoteText( lMoveNote );
      NextLine();
      SetMaxWidth( w );

      WidthComment( lComment );

      lComment = oMoveAnno.GetComment( PbnCommentAdmin.TYPE_NOTE_REF );
      WidthComment( lComment );
    }

    lSuffix = oMoveAnno.GetSuffix();
    if ( lSuffix.IsValid() )
    {
      w = 2;
      if ( oCall != null )
      {
        w += WidthCall( oCall );
        w += WidthSuffix( lSuffix, PbnGen.SECTION_AUCTION );
      }
      else
      {
        w += WidthCard( oCard );
        w += WidthSuffix( lSuffix, PbnGen.SECTION_PLAY );
      }
      NextLine();
      SetMaxWidth( w );

      lComment = oMoveAnno.GetComment( PbnCommentAdmin.TYPE_SUFFIX );
      WidthComment( lComment );
    }

    laNags = oMoveAnno.GetNags();
    int iNrNags = laNags.length;
    if ( iNrNags > 0 )
    {
      w = 2;
      if ( oCall != null )
      {
        w += WidthCall( oCall );
      }
      else
      {
        w += WidthCard( oCard );
      }

      for ( int iNag = 0; iNag < iNrNags; iNag++ )
      {
        w += WidthNag( laNags[ iNag ] );
      }
      NextLine();
      SetMaxWidth( w );

      lComment = oMoveAnno.GetComment( PbnCommentAdmin.TYPE_NAG );
      WidthComment( lComment );
    }
  }

  private int                   WidthNag(
  PbnNag                        oNag )
  {
    String                      lString = " $" + oNag.Get();

    return mFontMetrics.stringWidth( lString );
  }

  private int                   WidthNoteRef(
  PbnNote                       oNote )
  {
    String                      lString = " (" + oNote.Get() + ")";

    return mFontMetrics.stringWidth( lString );
  }

  private int                   WidthNoteText(
  PbnMoveNote                   oMoveNote )
  {
    String                      lString = oMoveNote.GetTagValue();

    return mFontMetrics.stringWidth( lString );
  }

  private void                  WidthSectionTags(
  Enumeration                   oeTags )
  {
    PbnTagId                    lTagId;
    PbnComment                  lComment;

    while ( oeTags.hasMoreElements()  )
    {
      lTagId = (PbnTagId) oeTags.nextElement();
      lComment = mGameTags.GetTagComment( lTagId );
      WidthComment( lComment );
    }
  }

  private void                  WidthSection(
  int                           iSection )
  {
    PbnComment                  lComment = null;
    PbnTagId                    lTagId = new PbnTagId();

    switch ( iSection )
    {
    case PbnGen.SECTION_IDENT:
      lComment = mGameTags.GetGameComment();
      WidthComment( lComment );

      WidthSectionTags( PbnTagId.GetMtsTags() );
      WidthSectionTags( PbnTagId.GetOptionalTags() );
      break;
    case PbnGen.SECTION_AUCTION:
      NextLine();
      lComment = mGameTags.GetTagComment( new PbnTagId( PbnTagId.AUCTION ) );
      WidthComment( lComment );
      break;
    case PbnGen.SECTION_PLAY:
      NextLine();
      lComment = mGameTags.GetTagComment( new PbnTagId( PbnTagId.PLAY ) );
      WidthComment( lComment );
      break;
    case PbnGen.SECTION_SUPPL:
      WidthSectionTags( PbnTagId.GetSupplementalTags() );
      break;
    }
  }

  private int                   WidthSuffix(
  PbnSuffix                     oSuffix,
  int                           iSection )
  {
    int                         iIndex = oSuffix.Get();

    if ( iSection != PbnGen.SECTION_AUCTION )
    {
      iIndex += PbnSuffix.NUMBER-1;
    }

    return mFontMetrics.stringWidth( " " + maNagTexts[ iIndex ] );
  }
}
