/*
 * File   :     GibProcess.java
 * Author :     Tis Veugen
 * Date   :     2005-01-22
 * PBN    :     2.0
 *
 * History
 * -------
 * 2005-01-22 Adapted for GIB Version 6.1.3
 */

import java.io.*;
import java.util.*;

class GibProcess
{
  public static final String    ArgLine = "Enter argument line: ";
  public static final String    CmdLine = "Enter command: ";

  private Process               mProcess = null;
  private BufferedReader        mReader;
  private BufferedWriter        mWriter;
  private int                   mNrTricks;

  public                        GibProcess()
  {
/*    try
    {
      mProcess = Runtime.getRuntime().exec( "bridge" );
      mReader = new BufferedReader(
                new InputStreamReader(
                mProcess.getInputStream() ) );
      mWriter = new BufferedWriter(
                new OutputStreamWriter(
                mProcess.getOutputStream() ) );
      FlushArgLine();
    }
    catch ( IOException oEvent )
    {
      mProcess = null;
    }
*/
  }

  public void                   Destroy()
  {
    if ( mProcess != null )
    {
      mProcess.destroy();
      mProcess = null;
    }
  }

  public void                   FlushArgLine()
  {
    ReadUntil( ArgLine );
  }

  public void                   FlushCmdLine()
  {
    ReadUntil( CmdLine );
  }

  public String                 ReadUntil(
  String                        oString )
  {
    char                        c;
    StringBuffer                lSb = new StringBuffer();
    String                      lS;
    int                         m;

    do
    {
      try
      {
        m = mReader.read();
      }
      catch ( IOException oEvent )
      {
        oEvent.printStackTrace();
        return null;
      }

      if ( m == -1 )
      {
        return null;
      }
      c = (char) m;
      lSb.append( c );
      lS = lSb.toString();
    } while ( ! lS.endsWith( oString ) );

    DebugPrint( "R>" + lS + "<" );
    lS = lS.substring( 0, lS.length() - oString.length() );
    return lS;
  }

  public String                 ReadLn()
  {
    return ReadUntil( "\n" );
  }

  public void                   Write(
  String                        oString )
  {
    DebugPrint( "W>" + oString + "<" );
    try
    {
      mWriter.write( oString, 0, oString.length() );
    }
    catch ( IOException oEvent )
    {
      oEvent.printStackTrace();
    }
  }

  public void                   WriteLn(
  String                        oString )
  {
    Write( oString + "\n" );
    try
    {
      mWriter.flush();
    }
    catch ( IOException oEvent )
    {
      oEvent.printStackTrace();
    }
  }

  private String []             ParseCards(
  String                        oString )
  {
    Vector                      lVector = new Vector();
    int                         iEol;

    for ( ; ; )
    {
      iEol = oString.indexOf( "\n" );
      if ( iEol <= 1 )
      {
        break;
      }
      char c1 = oString.charAt( 3 );
      int nt = c1 - '0';
      char c2 = oString.charAt( 4 );
      if ( c2 != ' ' )
      {
        nt = 10 * nt + c2 - '0';
      }
      if ( nt == mNrTricks )
      {
          lVector.addElement( oString.substring( 0, iEol ) );
      }
      oString = oString.substring( iEol+1 );
    }

    String []                   laCards = new String[ lVector.size() ];

    lVector.copyInto( laCards );
    return laCards;
  }

  public String []            GetOptCards(
  String                      oFilename )
  {

    try
    {
      mProcess = Runtime.getRuntime().exec( "bridge" + " -dw " + oFilename  );
      mReader = new BufferedReader(
                new InputStreamReader(
                mProcess.getInputStream() ) );
      mWriter = new BufferedWriter(
                new OutputStreamWriter(
                mProcess.getOutputStream() ) );
    }
    catch ( IOException oEvent )
    {
      mProcess = null;
    }


    if ( mProcess == null )
    {
      return null;
    }

    /*
     * Skip GIB version info, and
     * empty line
     */
    ReadLn();
    ReadLn();
    String                lTricks = ReadLn();
    if ( lTricks == null )
    {
      return null;
    }

    char                  c = lTricks.charAt( 0 );

    if ( c >= 'A' )
    {
      mNrTricks = 10 + c - 'A';
    }
    else
    {
      mNrTricks = c - '0';
    }

    FlushCmdLine();
    WriteLn( "p" );

    String lCardData = ReadUntil( CmdLine );
    String []           laCards = ParseCards( lCardData );

    for ( int i = 0; i < laCards.length; i++ )
    {
      DebugPrint( laCards[ i ] );
    }

    WriteLn( "q" );
    FlushArgLine();

    return laCards;
  }

  public int                    GetOptNrTricks()
  {
    return mNrTricks;
  }

  private void                  DebugPrint(
  String                        oString )
  {
//  System.out.println( oString );
  }

  public static void            main(
  String []                     args )
  {
    GibProcess                  mGibProcess = new GibProcess();
    String                      Filename = "TV_1.dd";

    if ( args.length > 0 )
    {
      Filename = args[0];
    }

    String []           laCards;

    laCards = mGibProcess.GetOptCards( "TV_1.dd" );
    laCards = mGibProcess.GetOptCards( "TV_2.dd" );
    laCards = mGibProcess.GetOptCards( "TVa_0.dd" );

    mGibProcess.Destroy();
  }
}
