/*
 * File   :     PanelArea.java
 * Author :     Tis Veugen
 * Date   :     2002-02-22
 * PBN    :     2.0
 *
 * History
 * -------
 * 1999-10-10 Check creating images, catch Throwable
 * 2002-02-22 Added table display
 */

import java.net.URL;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

public class PanelArea extends Panel
        implements AdjustmentListener, ComponentListener
{
  private static final int      EMPTY = 0;
  private static final int      FILE  = 1;
  private static final int      ANNO  = 2;

  public final char             CHAR_BACKSLASH = '\\';

  private int                   mWhat;
  private String []             maGameTexts;

  private Point                 mAnnoScroll;
  private Point                 mFileScroll;
  private Point                 mTableScroll;
  private Scrollbar             mHScroller;
  private Scrollbar             mVScroller;
  private CanvasArea            mCanvasArea;

  private Font                  mFont = new Font( "Monospaced", Font.PLAIN, 14 );
  private FontMetrics           mFontMetrics;
  private int                   mHeight;
  private int                   mYImage;
  private Image                 mFileImage = null;
  private Image                 mAnnoImage;
  private Image                 mTableImage;
  private Dimension             mFileSize;
  private Dimension             mAnnoSize;
  private Dimension             mTableSize;
  private AnnoRange             mAnnoPosition;
  private AnnoRange             mAnnoLast;

  public			PanelArea(
  PbnVwHndlr			oPbnVwHndlr )
  {
    setBackground( Color.white );
    setFont( mFont );

    Toolkit                     tk = Toolkit.getDefaultToolkit();
    mFontMetrics = tk.getFontMetrics( mFont );
    mHeight = mFontMetrics.getHeight();
    mYImage = mHeight - mFontMetrics.getAscent();

    mWhat = PanelScreen.ANNO_EMPTY;
    mAnnoScroll  = new Point();
    mFileScroll  = new Point();
    mTableScroll = new Point();
    mCanvasArea = new CanvasArea();
    mHScroller = new Scrollbar( Scrollbar.HORIZONTAL
                              , 0
                              , 10
                              , 0
                              , 0 );
    mHScroller.setUnitIncrement( 8 );
    mHScroller.addAdjustmentListener( this );

    mVScroller = new Scrollbar( Scrollbar.VERTICAL
                              , 0
                              , 10
                              , 0
                              , 0 );
    mVScroller.setUnitIncrement( mHeight );
    mVScroller.addAdjustmentListener( this );
    
    setLayout( new BorderLayout() );
    add( mVScroller, "East" );
    add( mHScroller, "South" );
    add( mCanvasArea, "Center" );

    addComponentListener( this );
  }

  public void                   adjustmentValueChanged(
  AdjustmentEvent               oEvent )
  {
    int			        iNewY = mVScroller.getValue();
    int			        iNewX = mHScroller.getValue();
    Dimension                   lSize;
    Point                       lScroll;

    switch ( mWhat )
    {
    case PanelScreen.ANNO_FILE:
      mFileScroll = new Point( iNewX, iNewY );
      RefreshScroll( mFileSize, mFileScroll );
      break;
    case PanelScreen.ANNOTATION:
      mAnnoScroll = new Point( iNewX, iNewY );
      RefreshScroll( mAnnoSize, mAnnoScroll );
      break;
    case PanelScreen.ANNO_TABLE:
      mTableScroll = new Point( iNewX, iNewY );
      RefreshScroll( mTableSize, mTableScroll );
      break;
    }
  }

  public void                   componentHidden(
  ComponentEvent                oEvent )
  {
  }

  public void                   componentMoved(
  ComponentEvent                oEvent )
  {
  }

  public void                   componentResized(
  ComponentEvent                oEvent )
  {
    switch ( mWhat )
    {
    case PanelScreen.ANNO_FILE:
      RefreshScroll( mFileSize, mFileScroll );
      break;
    case PanelScreen.ANNOTATION:
      RefreshScroll( mAnnoSize, mAnnoScroll );
      break;
    case PanelScreen.ANNO_TABLE:
      RefreshScroll( mTableSize, mTableScroll );
      break;
    }
  }

  public void                   componentShown(
  ComponentEvent                oEvent )
  {
    componentResized( oEvent );
  }

  public int 			GetAreaWidth()
  {
    return mCanvasArea.getSize().width;
  }

  public void			SetGameTexts(
  String []                     oaStrings )
  {
    mFileSize  = GetFileDimension( oaStrings );
    try
    {
      mFileImage = createImage( mFileSize.width, mFileSize.height );
      RenderFile( mFileImage, oaStrings );
    }
        catch ( Throwable E )
    {
      mFileImage = null;
    }

    if ( mFileImage != null )
    {
      return;
    }

    String []                   laStrings = { "PBN File too big" };

    mFileSize  = GetFileDimension( laStrings );
    try
    {
      mFileImage = createImage( mFileSize.width, mFileSize.height );
      RenderFile( mFileImage, laStrings );
    }
    catch ( Throwable E )
    {
      mFileImage = null;
    }
  }

  public void                   SetTableTexts(
  String []                     oaStrings )
  {
    mTableSize  = GetFileDimension( oaStrings );
    try
    {
      mTableImage = createImage( mTableSize.width, mTableSize.height );
      RenderFile( mTableImage, oaStrings );
    }
        catch ( Throwable E )
    {
      mTableImage = null;
    }

    if ( mTableImage != null )
    {
      return;
    }

    String []                   laStrings = { "PBN Table too big" };

    mTableSize  = GetFileDimension( laStrings );
    try
    {
      mTableImage = createImage( mTableSize.width, mTableSize.height );
      RenderFile( mTableImage, laStrings );
    }
    catch ( Throwable E )
    {
      mTableImage = null;
    }
  }

  public void			ClearAnno()
  {
    mWhat = PanelScreen.ANNO_EMPTY;
    mHScroller.setValues( 0, 0, 0, 0 );
    mVScroller.setValues( 0, 0, 0, 0 );
    mAnnoScroll = new Point();
    mFileScroll = new Point();
    mCanvasArea.SetHeight( 0 );
    mCanvasArea.repaint();
  }

  private void                  MyWait(
  int                           iMilliSecs )
  {
    try
    {
      Thread.currentThread().sleep( iMilliSecs );
    }
    catch( InterruptedException e )
    {}
  }

  private void                  WaitForImage(
  Image                         oImage )
  {
    MediaTracker                lMediaTracker = new MediaTracker( this );

    try
    {
      lMediaTracker.addImage( oImage, 0 );
      lMediaTracker.waitForID( 0 );
    }
    catch( InterruptedException oEvent )
    {
      oEvent.printStackTrace();
    }
  }

  public void			SetAnno(
  int                           iAnnotation )
  {
    switch ( iAnnotation )
    {
    case PanelScreen.ANNO_FILE:
      AnnoFile();
      break;
    case PanelScreen.ANNOTATION:
      mAnnoPosition.clone( mAnnoLast );
      Annotation();
      break;
    case PanelScreen.ANNO_TABLE:
      AnnoTable();
      break;
    default:
      break;
    }
  }

  public void			SetAnnoRange(
  AnnoRange                     oAnnoRange )
  {
    mAnnoLast.clone( oAnnoRange );
    if ( mAnnoPosition.End < oAnnoRange.End )
    {
      mAnnoPosition.clone( oAnnoRange );

      if ( mWhat == PanelScreen.ANNOTATION )
      {
        Annotation();
      }
    }
  }

  public void			SetAnnoImage(
  Image                         oImage )
  {
    mAnnoImage = oImage;
    WaitForImage( oImage );
    mAnnoSize = new Dimension( oImage.getWidth( this )
                             , 1 );

    mAnnoPosition = new AnnoRange( 0, 1 );
    mAnnoLast     = new AnnoRange( 0, 1 );
  }

  private void			Annotation()
  {
    mWhat = PanelScreen.ANNOTATION;
    mAnnoSize.height = mAnnoPosition.End + 4;
    mAnnoScroll = new Point( 0, mAnnoPosition.Begin );
    RefreshAll( mAnnoImage
              , mAnnoSize
              , mAnnoScroll );
  }

  private void                        AnnoFile()
  {
    if ( mFileImage != null )
    {
      mWhat = PanelScreen.ANNO_FILE;
      RefreshAll( mFileImage
                , mFileSize
                , mFileScroll );
    }
  }

  private void                        AnnoTable()
  {
    mWhat = PanelScreen.ANNO_TABLE;
    RefreshAll( mTableImage
              , mTableSize
              , mTableScroll );
  }

  private void			RefreshAll(
  Image                         oImage,
  Dimension                     oSize,
  Point                         oScroll )
  {
    mCanvasArea.SetImage( oImage );
    mCanvasArea.SetHeight( oSize.height );
    RefreshScroll( oSize, oScroll );
  }

  private void			RefreshScroll(
  Dimension                     oSize,
  Point                         oScroll )
  {
    mCanvasArea.ScrollTo( oScroll );
    mCanvasArea.repaint();

    /*
     * Value   = scroll value
     * Visible = viewport size
     * Minimum = 0
     * Maximum = image size - viewport size
     */
    Dimension                   lSize = mCanvasArea.getSize();
    mVScroller.setBlockIncrement( lSize.height - mHeight );
    mVScroller.setValues( oScroll.y
                        , lSize.height
                        , 0
                        , oSize.height );

    mHScroller.setBlockIncrement( lSize.width - 8 );
    mHScroller.setValues( oScroll.x
                        , lSize.width
                        , 0
                        , oSize.width );
  }

  private Dimension             GetFileDimension(
  String []                     oaStrings )
  {
    int                         iWidth  = 0;
    int                         iHeight = 0;
    int                         iNrLines = oaStrings.length;
    int                         w;

    for ( int i = 0; i < iNrLines; i++ )
    {
      w = oaStrings[ i ].length() * mFontMetrics.charWidth( 'A' );
      if ( iWidth < w )
      {
        iWidth = w;
      }
    }
    int h = iNrLines * mHeight
          + mFontMetrics.getMaxDescent();
    return new Dimension( 2 + iWidth + 2, 2 + h + 2 );
  }

  private void                  RenderFile(
  Image                         oImage,
  String []                     oaStrings )
  {
    int                         iNrLines = oaStrings.length;
    int                         y = 2;
    Graphics                    lGraphics = oImage.getGraphics();

    try
    {
      for ( int i = 0; i < iNrLines; i++ )
      {
        lGraphics.drawString( oaStrings[ i ], 2, y + mHeight );
        y += mHeight;
      }
    }
    finally
    {
      lGraphics.dispose();
    }
  }
}

class CanvasArea extends Canvas
{
  private Image                 mImage;
  private int                   mWidth;
  private int                   mHeight;
  private Point                 mScroll;

  public                        CanvasArea()
  {
    mImage = null;
    mWidth  =
    mHeight = 0;
  }

  public void			paint(
  Graphics			oGraphics )
  {
    Dimension                   lSize = getSize();
    oGraphics.setColor( Color.white );
    oGraphics.fillRect( 0, 0
                      , lSize.width, lSize.height );

    if ( mHeight > 0 )
    {
      int                       lH;
      int                       iImageY;

      if ( mHeight <= lSize.height )
      {
        lH = mHeight;
        iImageY = 0;
      }
      else
      {
        lH = lSize.height;
        iImageY = mHeight - lSize.height;
        if ( mScroll.y < iImageY )
        {
          iImageY = mScroll.y;
        }
      }

      int                       lW;
      int                       iImageX;

      if ( mWidth <= lSize.width )
      {
        lW = mWidth;
        iImageX = 0;
      }
      else
      {
        lW = lSize.width;
        iImageX = mWidth - lSize.width;
        if ( mScroll.x < iImageX )
        {
          iImageX = mScroll.x;
        }
      }

      oGraphics.drawImage( mImage
                         , 0, 0
                         , lW, lH
                         , iImageX, iImageY
                         , iImageX + lW, iImageY + lH
                         , this );
    }
  }

  public void                   SetHeight(
  int                           iHeight )
  {
    mHeight = iHeight;
  }

  public void                   SetImage(
  Image                         oImage )
  {
    mImage = oImage;
    mWidth = mImage.getWidth( this );
    mHeight = mImage.getHeight( this );
  }

  public void                   ScrollTo(
  Point                         oScroll )
  {
    mScroll = oScroll;
  }
}
