/*
 * File   :     PanelHist.java
 * Author :     Tis Veugen
 * Date   :     1999-01-13
 * PBN    :     1.0
 */

import java.awt.*;

public class PanelHist extends Panel
{
  private PanelColumn []        maColumns;

  public			PanelHist()
  {
    setBackground( Color.blue );
    setFont( PanelScreen.FontCards );

    maColumns = new PanelColumn[ PbnSide.NUMBER ];
    for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
    {
      maColumns[ iSide ] = new PanelColumn( new PbnSide( iSide ) );
    }

    // 1 row of panels
    GridLayout			lGl = new GridLayout( 1, PbnSide.NUMBER, 2, 0 );
    setLayout( lGl );

    add( maColumns[ PbnSide.WEST  ] );
    add( maColumns[ PbnSide.NORTH ] );
    add( maColumns[ PbnSide.EAST  ] );
    add( maColumns[ PbnSide.SOUTH ] );
  }

  public Insets			getInsets()
  {
    return new Insets( 2, 2, 2, 2 );
  }

  public void			ClearAll()
  {
    for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
    {
      maColumns[ iSide ].ClearAll();
    }
  }

  public void			ClearAll(
  PbnSide			oSide )
  {
    maColumns[ oSide.Get() ].ClearAll();
  }

  public void			Clear(
  PbnSide			oSide,
  int                           iRow )
  {
    maColumns[ oSide.Get() ].Clear( iRow );
  }

  public void			Bar(
  PbnSide			oSide,
  int                           iRow )
  {
    maColumns[ oSide.Get() ].Bar( iRow );
  }

  public void			Question(
  PbnSide			oSide,
  int                           iRow )
  {
    maColumns[ oSide.Get() ].Question( iRow );
  }

  public void			Show(
  int                           iRow,
  PbnCall			oCall,
  char                          cIllegal,
  int                           iNote )
  {
    maColumns[ oCall.GetSide().Get() ].Show( iRow, oCall, cIllegal, iNote );
  }

  public void			Show(
  int                           iRow,
  PbnPlayCard			oPlayCard,
  char                          cIllegal,
  int                           iNote )
  {
    maColumns[ oPlayCard.GetSide().Get() ].Show( iRow, oPlayCard.GetCard()
                                               , cIllegal, iNote );
  }
}

class PanelColumn extends Panel
{
  public static final int       ROWS = 13;
  private CanvasName            mCanvasName;
  private CanvasCell []         maCanvasCells;

  public			PanelColumn(
  PbnSide                       oSide )
  {
    mCanvasName = new CanvasName( oSide.toString() );
    maCanvasCells = new CanvasCell[ ROWS ];

    // 1 column of panels
    GridLayout			lGl = new GridLayout( 1+ROWS, 1, 0, 0 );
    setLayout( lGl );

    add( mCanvasName );
    for ( int iRow = 0; iRow < ROWS; iRow++ )
    {
      maCanvasCells[ iRow ] = new CanvasCell();
      add( maCanvasCells[ iRow ] );
    }
  }

  public void			ClearAll()
  {
    for ( int iRow = 0; iRow < ROWS; iRow++ )
    {
      Clear( iRow );
    }
  }

  public void			Clear(
  int                           iRow )
  {
    maCanvasCells[ iRow ].Clear();
  }

  public void			Bar(
  int                           iRow )
  {
    maCanvasCells[ iRow ].Bar();
  }

  public void			Question(
  int                           iRow )
  {
    maCanvasCells[ iRow ].Question();
  }

  public void			Show(
  int                           iRow,
  PbnCall			oCall,
  char                          cIllegal,
  int                           iNote )
  {
    maCanvasCells[ iRow ].Show( oCall, cIllegal, iNote );
  }

  public void			Show(
  int                           iRow,
  PbnCard			oCard,
  char                          cIllegal,
  int                           iNote )
  {
    maCanvasCells[ iRow ].Show( oCard, cIllegal, iNote );
  }
}

class CanvasName extends Canvas
{
  private static final int	WIDTH  = 56;
  private static final int	HEIGHT = 2+16+2;//     2+14+2;
  private String                mName;

  public                        CanvasName(
  String                        oName )
  {
    mName = oName;
    setSize( WIDTH, HEIGHT );
    setFont( new Font( "Dialog", Font.BOLD, 16 ) );
  }

  public void			paint(
  Graphics			oGraphics )
  {
    Dimension                   lDimension = getSize();
    int                         lW = lDimension.width;
    int                         lH = lDimension.height;

    Color	                lColorBG;
    if ( mName.length() == 4 )
    {
      lColorBG = new Color( 0, 255, 128 );
    }
    else
    {
      lColorBG = new Color( 255, 0, 128 );
    }
    oGraphics.setColor( lColorBG );
    oGraphics.fillRect( 0, 0, lW, lH );

    oGraphics.setColor( Color.white );
    oGraphics.fillRect( 4, 2, lW-4-4, lH-2-2 );

    FontMetrics                 lFontMetrics = oGraphics.getFontMetrics();
    int                         w = lFontMetrics.stringWidth( mName );

    oGraphics.setColor( Color.black );
    oGraphics.drawString( mName, 4 + (lW-4-4-w)/2, lH-2-2 );
  }
}

/*
 * s : Spades
 * h : Hearts
 * d : Diamonds
 * c : Clubs
 * ^ : illegal
 * I : Insufficient
 * S : Skip
 * L : Lead
 * R : Revoke
 *
 * J : Jack
 * Q : Queen
 * K : King
 * A : Ace
 */
class CanvasCell extends Canvas
{
  private static final int	WIDTH  = 56;
  private static final int	HEIGHT = 16;

  private static final int	CLEAR    = 0;
  private static final int	CALL     = 1;
  private static final int	CARD     = 2;
  private static final int	QUESTION = 3;

  private int                   mWhat = CLEAR;
  private PbnCall               mCall;
  private PbnCard               mCard;
  private char                  mcIllegal = '\0';
  private int                   mNote = 0;
  private boolean               mbBar = false;
  private int                   lW;
  private int                   lH;
  private int                   lHS;    // lH for strings

  public                        CanvasCell()
  {
    setSize( WIDTH, 2+HEIGHT );
  }

  public void			Clear()
  {
    mWhat = CLEAR;
    mbBar = false;
    repaint();
  }

  public void			Bar()
  {
    mbBar = true;
    repaint();
  }

  public void			Question()
  {
    mWhat = QUESTION;
    repaint();
  }

  public void			Show(
  PbnCall			oCall,
  char                          cIllegal,
  int                           iNote )
  {
    mWhat = CALL;
    mCall = oCall;
    mcIllegal = cIllegal;
    mNote = iNote;
    repaint();
  }

  public void			Show(
  PbnCard			oCard,
  char                          cIllegal,
  int                           iNote )
  {
    mWhat = CARD;
    mCard = oCard;
    mcIllegal = cIllegal;
    mNote = iNote;
    repaint();
  }

  public void			paint(
  Graphics			oGraphics )
  {
    int                         w1, w2, w, x;

    Dimension                   lDimension = getSize();
    lW = lDimension.width;
    lH = lDimension.height;
    lHS = lH-3;

    switch ( mWhat )
    {
    case CLEAR:
      oGraphics.setColor( Color.blue );
      oGraphics.fillRect( 0, 0, lW, lH );
      if ( mbBar )
      {
        oGraphics.setColor( Color.white );
        oGraphics.fillRect( 0, 0, lW, lH-HEIGHT );
      }
      break;

    case CALL:
      oGraphics.setColor( Color.white );
      oGraphics.fillRect( 0, lH-HEIGHT, lW, HEIGHT );

      w1 = WidthIllegal( oGraphics );
      w2 = WidthCall( oGraphics );
      w = w1 + w2;

      x = (lW-w)/2;
      DrawIllegal( oGraphics, x );
      x += w1;
      DrawCall( oGraphics, x );
      DrawNote( oGraphics );
      break;

    case CARD:
      oGraphics.setColor( Color.white );
      if ( mbBar )
      {
        oGraphics.fillRect( 0, 0, lW, lH );
      }
      else
      {
        oGraphics.fillRect( 0, lH-HEIGHT, lW, HEIGHT );
      }

      w1 = WidthIllegal( oGraphics );
      w2 = WidthCard( oGraphics );
      w = w1 + w2;

      x = (lW-w)/2;
      DrawIllegal( oGraphics, x );
      x += w1;
      DrawCard( oGraphics, x );
      DrawNote( oGraphics );
      break;

    case QUESTION:
      FontMetrics               lFontMetrics = oGraphics.getFontMetrics();

      oGraphics.setColor( Color.white );
      oGraphics.fillRect( 0, lH-HEIGHT, lW, HEIGHT );

      oGraphics.setColor( Color.black );
      w = lFontMetrics.stringWidth( "?" );
      oGraphics.drawString( "?", (lW-w)/2, lHS );
      break;
    }
  }

  private int                   WidthIllegal(
  Graphics			oGraphics )
  {
    if ( mcIllegal == '\0' )
    {
      return 0;
    }

    FontMetrics                 lFontMetrics = oGraphics.getFontMetrics();

    return lFontMetrics.stringWidth( "^" + mcIllegal ) + 2;
  }

  private int                   WidthCall(
  Graphics			oGraphics )
  {
    String                      lString = null;
    int                         wTrump = 0;

    switch ( mCall.GetType() )
    {
    case PbnCall.TYPE_PASS:
    case PbnCall.TYPE_DBL:
    case PbnCall.TYPE_RDB:
      lString = mCall.toString();
      break;

    case PbnCall.TYPE_REAL:
    case PbnCall.TYPE_INSUFF:
      lString = "" + mCall.GetNrTricks();
      Image mTrumpImage = MyImages.Get( mCall.GetTrump() );
      wTrump = 2 + mTrumpImage.getWidth( this );
      break;

    default:
      break;
    }

    if ( lString == null )
    {
      return 0;
    }

    FontMetrics                 lFontMetrics = oGraphics.getFontMetrics();

    return lFontMetrics.stringWidth( lString ) + wTrump;
  }

  private int                   WidthCard(
  Graphics			oGraphics )
  {
    PbnSuit                     lSuit = mCard.GetSuit();
    PbnRank                     lRank = mCard.GetRank();
    String                      lString;
    int                         w;
    FontMetrics                 lFontMetrics = oGraphics.getFontMetrics();

    if ( ! ( lSuit.IsValid()
          && lRank.IsNormal() ) )
    {
      lString = "-";
      w = lFontMetrics.stringWidth( lString );
    }
    else
    {
      Image mSuitImage = MyImages.Get( lSuit );
      w = mSuitImage.getWidth( this ) + 2;

      lString = lRank.toCharacter();
      w += lFontMetrics.stringWidth( lString );
    }

    return w;
  }

  private void                  DrawIllegal(
  Graphics			oGraphics,
  int                           x )
  {
    if ( mcIllegal != '\0' )
    {
      oGraphics.setColor( PanelScreen.ColorPink );
      oGraphics.drawString( "^" + mcIllegal, x, lHS );
    }
  }

  private void                  DrawCall(
  Graphics			oGraphics,
  int                           x )
  {
    String                      lString;
    FontMetrics                 lFontMetrics = oGraphics.getFontMetrics();

    oGraphics.setColor( Color.black );
    switch ( mCall.GetType() )
    {
    case PbnCall.TYPE_PASS:
    case PbnCall.TYPE_DBL:
    case PbnCall.TYPE_RDB:
      lString = mCall.toString();
      oGraphics.drawString( lString, x, lHS );
      break;

    case PbnCall.TYPE_REAL:
    case PbnCall.TYPE_INSUFF:
      lString = "" + mCall.GetNrTricks();
      oGraphics.drawString( lString, x, lHS );
      x += lFontMetrics.stringWidth( lString ) + 2;

      Image mTrumpImage = MyImages.Get( mCall.GetTrump() );
      oGraphics.drawImage( mTrumpImage, x, lH-HEIGHT, this );
      break;

    default:
      break;
    }
  }

  private void                  DrawCard(
  Graphics			oGraphics,
  int                           x )
  {
    String                      lString;
    PbnSuit                     lSuit = mCard.GetSuit();
    PbnRank                     lRank = mCard.GetRank();

    oGraphics.setColor( Color.black );
    if ( ! ( lSuit.IsValid()
          && lRank.IsNormal() ) )
    {
      lString = "-";
      oGraphics.drawString( lString, x, lHS );
    }
    else
    {
      Image mSuitImage = MyImages.Get( lSuit );
      oGraphics.drawImage( mSuitImage, x, lH-HEIGHT, this );
      x += mSuitImage.getWidth( this ) + 2;

      lString = lRank.toCharacter();
      oGraphics.drawString( lString, x, lHS );
    }
  }

  private void                  DrawNote(
  Graphics			oGraphics )
  {
    String                      lString;
    FontMetrics                 lFontMetrics = oGraphics.getFontMetrics();

    if ( mNote != 0 )
    {
      oGraphics.setFont( PanelScreen.FontNote );
      oGraphics.setColor( PanelScreen.ColorDarkGreen );
      lString = "" + mNote;
      int w = lFontMetrics.stringWidth( lString );
      oGraphics.drawString( lString, lW-1-w, 14 );
      // Restore font
      oGraphics.setFont( PanelScreen.FontCards );
    }
  }
}

