/*
 * File   :     PanelScreen.java
 * Author :     Tis Veugen
 * Date   :     2004-08-23
 * PBN    :     2.0
 *
 * History
 * -------
 * 2001-09-20 Made uniform files
 * 2002-02-22 Added table display
 * 2004-08-23 Added Rotate()
 */

import java.awt.*;

public class PanelScreen extends GridPanel
{
  static public final Font  	FontCards = new Font( "Dialog", Font.BOLD, 16 );
  static public final Font  	FontNames = new Font( "SansSerif", Font.PLAIN, 16 );
  static public final Font  	FontNote  = new Font( "Dialog", Font.PLAIN, 10 );
  static public final Color 	ColorPink      = new Color( 255, 0, 255 );
  static public final Color 	ColorDarkGreen = new Color( 0, 128, 0 );

  static public final int       ANNO_EMPTY = 0;
  static public final int       ANNO_FILE  = 1;
  static public final int       ANNOTATION = 2;
  static public final int       ANNO_TABLE = 3;

  private PanelTable		mPanelTable;
  private PanelHist		mPanelHist;
  private PanelArea		mPanelArea;
  private PanelSet		mPanelSet;

  public			PanelScreen(
  PbnVwHndlr			oPbnVwHndlr )
  {
    if ( ! PbnViewer.mbApplet )
    {
      MyImages.Load( this );
    }

    mPanelTable = new PanelTable();
    mPanelHist  = new PanelHist();
    mPanelArea  = new PanelArea( oPbnVwHndlr );
    mPanelSet   = new PanelSet( oPbnVwHndlr );

    GridBagLayout		lGbl = new GridBagLayout();
    GridBagConstraints		lGbc = new GridBagConstraints();

    setLayout( lGbl );

    lGbc.anchor = GridBagConstraints.CENTER;
    lGbc.fill   = GridBagConstraints.BOTH;
    lGbc.weightx =
    lGbc.weighty = 0.0;
    addGrid( mPanelTable, lGbc, 0, 0, 1, 1 );
    addGrid( mPanelHist,  lGbc, 1, 0, 1, 1 );
    addGrid( mPanelSet,   lGbc, 1, 1, 1, 1 );
    lGbc.weightx =
    lGbc.weighty = 1.0;
    addGrid( mPanelArea,  lGbc, 0, 1, 1, 1 );
  }

  public void			SetBoard(
  String			oBoardNr )
  {
    mPanelTable.SetBoard( oBoardNr );
  }

  public void			SetSituation(
  PbnSituation			oSituation )
  {
    mPanelTable.SetSituation( oSituation );
  }

  public void			ClearContract()
  {
    mPanelTable.ClearContract();
  }

  public void			SetContract(
  PbnContract			oContract )
  {
    mPanelTable.SetContract( oContract );
  }

  public void			ClearResult()
  {
    mPanelTable.ClearResult();
  }

  public void			SetResult(
  PbnResult			oResult )
  {
    mPanelTable.SetResult( oResult );
  }

  public void			ClearNrTricks()
  {
    mPanelTable.ClearNrTricks();
  }

  public void			SetNrTricks(
  int				iNrTricksNS,
  int				iNrTricksEW )
  {
    mPanelTable.SetNrTricks( iNrTricksNS, iNrTricksEW );
  }

  public void			PlayCard(
  PbnPlayCard                   oPlayCard,
  char                          cIllegal,
  int                           iNote )
  {
    mPanelTable.PlayCard( oPlayCard
                        , cIllegal
                        , iNote );
  }

  public void			PlayClearAll()
  {
    mPanelTable.PlayClearAll();
  }

  public void			PlayClearSide(
  PbnSide			oSide )
  {
    mPanelTable.PlayClearSide( oSide );
  }

  public void			PlayQuestion(
  PbnSide			oSide )
  {
    mPanelTable.PlayQuestion( oSide );
  }

  public void			SetName(
  PbnSide			oSide,
  String			oName )
  {
    mPanelTable.SetName( oSide, oName );
  }

  public void			ClearHand(
  PbnSide			oSide )
  {
    mPanelTable.ClearHand( oSide );
  }

  public void			SetHand(
  PbnSide			oSide,
  PbnHand			oHand )
  {
    mPanelTable.SetHand( oSide, oHand );
  }

  public void			SetHand(
  PbnSide			oSide,
  PbnHand			oHand,
  PbnHand			oOptHand )
  {
    mPanelTable.SetHand( oSide, oHand, oOptHand );
  }

  public void			ClearOptNrTricks()
  {
    mPanelTable.ClearOptNrTricks();
  }

  public void			SetOptNrTricks(
  PbnSide			oSide,
  int                           iNrTricks )
  {
    mPanelTable.SetOptNrTricks( oSide, iNrTricks );
  }

  public void			TableHidden(
  boolean []			abHiddenSide )
  {
    mPanelTable.SetHidden( abHiddenSide );
  }

  public void			HistClearAll()
  {
    mPanelHist.ClearAll();
  }

  public void			HistClearAll(
  PbnSide			oSide )
  {
    mPanelHist.ClearAll( oSide );
  }

  public void			HistClear(
  PbnSide			oSide,
  int                           iRow )
  {
    mPanelHist.Clear( oSide, iRow );
  }

  public void			HistBar(
  PbnSide			oSide,
  int                           iRow )
  {
    mPanelHist.Bar( oSide, iRow );
  }

  public void			HistQuestion(
  PbnSide			oSide,
  int                           iRow )
  {
    mPanelHist.Question( oSide, iRow );
  }

  public void			HistShow(
  int                           iRow,
  PbnCall			oCall,
  char                          cIllegal,
  int                           iNote )
  {
    mPanelHist.Show( iRow, oCall, cIllegal, iNote );
  }

  public void			HistShow(
  int                           iRow,
  PbnPlayCard			oPlayCard,
  char                          cIllegal,
  int                           iNote )
  {
    mPanelHist.Show( iRow, oPlayCard, cIllegal, iNote );
  }

  public void                   SetGameTexts(
  String []                     laStrings )
  {
    mPanelArea.SetGameTexts( laStrings );
  }

  public void                   SetTableTexts(
  String []                     laStrings )
  {
    mPanelArea.SetTableTexts( laStrings );
  }

  public void			ClearAnno()
  {
    mPanelArea.ClearAnno();
  }

  public void			SetAnno(
  int                           iAnnotation )
  {
    mPanelArea.SetAnno( iAnnotation );
  }

  public void			SetAnnoRange(
  AnnoRange                     oAnnoRange )
  {
    mPanelArea.SetAnnoRange( oAnnoRange );
  }

  public void			SetAnnoImage(
  Image                         oImage )
  {
    mPanelArea.SetAnnoImage( oImage );
  }

  public int 			GetAreaWidth()
  {
    return mPanelArea.GetAreaWidth();
  }

  public void			SetSection(
  int				iSection )
  {
    mPanelSet.SetSection( iSection );
  }

  public void			SetGameNr(
  int				iGameNr )
  {
    mPanelSet.SetGameNr( iGameNr );
  }

  public void			SetGameMax(
  int				iGameMax )
  {
    mPanelSet.SetGameMax( iGameMax );
  }

  public void			SetHidden(
  boolean []			abHiddenSide )
  {
    mPanelSet.SetHidden( abHiddenSide );
  }

  public void			Reset()
  {
    SetBoard( null );
    SetSituation( null );
    ClearContract();
    ClearResult();
    ClearNrTricks();
    PlayClearAll();

    PbnSide             lSide = new PbnSide( PbnSide.SOUTH );
    for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
    {
      ClearHand( lSide );
      SetName( lSide, "" );
      lSide.Next();
    }

    HistClearAll();
    SetGameMax( 0 );
    ClearAnno();
  }

  public void                   Rotate(
  int                           number )
  {
    mPanelTable.Rotate( number );
  }
}
