/*
 * File   :     PanelSet.java
 * Author :     Tis Veugen
 * Date   :     2004-08-23
 * PBN    :     1.0
 *
 * History
 * -------
 * 2004-08-23 More space for hidden hands checkboxes
 */

import java.awt.*;
import java.awt.event.*;

public class PanelSet extends GridPanel
{
  static public final Color	ColorBG = Color.white;
  private PanelControl		mPanelControl;
  private PanelSection		mPanelSection;
  private PanelGame		mPanelGame;
  private PanelHidden		mPanelHidden;

  private PanelFiles		mPanelFiles;

  public			PanelSet(
  PbnVwHndlr			oPbnVwHndlr )
  {
    setBackground( ColorBG );

    mPanelControl = new PanelControl( oPbnVwHndlr );
    mPanelSection = new PanelSection( oPbnVwHndlr );
    mPanelGame    = new PanelGame( oPbnVwHndlr );
    mPanelHidden  = new PanelHidden( oPbnVwHndlr );

    mPanelFiles   = new PanelFiles( oPbnVwHndlr );

    GridBagLayout		lGbl = new GridBagLayout();
    GridBagConstraints		lGbc = new GridBagConstraints();

    setLayout( lGbl );

    lGbc.anchor = GridBagConstraints.CENTER;
    lGbc.fill   = GridBagConstraints.BOTH;

    addGrid( mPanelControl, lGbc, 0, 0, 1, 1 );
    addGrid( mPanelSection, lGbc, 0, 1, 1, 1 );
    addGrid( mPanelGame   , lGbc, 0, 2, 1, 1 );
    addGrid( mPanelHidden , lGbc, 0, 3, 1, 1 );
//    addGrid( mPanelFiles  , lGbc, 0, 4, 1, 1 );
  }

  public void			SetSection(
  int				iSection )
  {
    mPanelSection.SetSection( iSection );
  }

  public void			SetGameNr(
  int				iGameNr )
  {
    mPanelGame.SetGameNr( iGameNr );
  }

  public void			SetGameMax(
  int				iGameMax )
  {
    mPanelGame.SetGameMax( iGameMax );
  }

  public void			SetHidden(
  boolean []			mbHiddenSide )
  {
    mPanelHidden.Set( mbHiddenSide );
  }
}

class PanelSection extends Panel
        implements ItemListener
{
  private Checkbox		mCBoxAuction;
  private Checkbox		mCBoxPlay;
  private PbnVwHndlr		mPbnVwHndlr;

  public			PanelSection(
  PbnVwHndlr			oPbnVwHndlr )
  {
    mPbnVwHndlr = oPbnVwHndlr;

    mCBoxAuction = new Checkbox( "Auction" );
    mCBoxPlay    = new Checkbox( "Play   " );

    // 1 row of panels
    GridLayout			lGl = new GridLayout( 1, 0, 2, 2 );
    setLayout( lGl );

    add( mCBoxAuction );
    add( mCBoxPlay );

    mCBoxAuction.addItemListener( this );
    mCBoxPlay.addItemListener( this );
  }

    public void			itemStateChanged(
    ItemEvent			oEvent )
    {
      int                       iSection;

      if ( oEvent.getItem().equals( "Auction" ) )
      {
        if ( mCBoxAuction.getState() )
        {
          mCBoxPlay.setState( false );
          iSection = PbnGen.SECTION_AUCTION;
        }
        else
        {
          iSection = PbnGen.SECTION_IDENT;
        }
      }
      else
      {
        if ( mCBoxPlay.getState() )
        {
          mCBoxAuction.setState( false );
          iSection = PbnGen.SECTION_PLAY;
        }
        else
        {
          iSection = PbnGen.SECTION_IDENT;
        }
      }

      mPbnVwHndlr.SetSection( iSection );
    }

  public void			SetSection(
  int				iSection )
  {
    switch ( iSection )
    {
    case PbnGen.SECTION_IDENT:
      mCBoxAuction.setState( false );
      mCBoxPlay.setState( false );
      break;
    case PbnGen.SECTION_AUCTION:
      mCBoxAuction.setState( true );
      mCBoxPlay.setState( false );
      break;
    case PbnGen.SECTION_PLAY:
      mCBoxAuction.setState( false );
      mCBoxPlay.setState( true );
      break;
    }
  }

  public Insets			getInsets()
  {
    return new Insets( 2, 16, 2, 4 );
  }

  public void			paint(
  Graphics			oGraphics )
  {
    Dimension			lDimension = getSize();

    setBackground( Color.white );
    oGraphics.drawRect( 1, 1, lDimension.width-2, lDimension.height-2 );
  }
}

class PanelGame extends GridPanel
        implements AdjustmentListener
{
  private Label			mLabelText;
  private Label			mLabelGameNr;
  private Scrollbar		mSbGameNr;
  private PbnVwHndlr		mPbnVwHndlr;

  public			PanelGame(
  PbnVwHndlr			oPbnVwHndlr )
  {
    mPbnVwHndlr = oPbnVwHndlr;

    mLabelText   = new Label( "Game Index" );
    mLabelGameNr = new Label( "" + 0 );
    mSbGameNr    = new Scrollbar( Scrollbar.HORIZONTAL
                                , 0
                                , 10
                                , 0
                                , 0 );

    GridBagLayout		lGbl = new GridBagLayout();
    GridBagConstraints		lGbc = new GridBagConstraints();

    setLayout( lGbl );

    lGbc.anchor = GridBagConstraints.CENTER;
    lGbc.fill   = GridBagConstraints.BOTH;

    addGrid( mLabelText,   lGbc, 0, 0, 2, 1 );
    addGrid( mLabelGameNr, lGbc, 2, 0, 1, 1 );
    addGrid( mSbGameNr,    lGbc, 0, 1, 3, 1 );

    mSbGameNr.addAdjustmentListener( this );
  }

    public void			adjustmentValueChanged(
    AdjustmentEvent		oEvent )
    {
      int			iGameNr = mSbGameNr.getValue();

      iGameNr--;
      SetGameNr( iGameNr );
      mPbnVwHndlr.SetGameIndex( iGameNr );
    }

  public void			SetGameNr(
  int				iGameNr )
  {
    iGameNr++;
    mLabelGameNr.setText( "" + iGameNr );
    mSbGameNr.setValue( iGameNr );
  }

  public void			SetGameMax(
  int				iGameMax )
  {
    mLabelGameNr.setText( "" + 0 );
    mSbGameNr.setValues( 0
                       , 10
                       , (iGameMax==0) ? 0 : 1
                       , (iGameMax==0) ? 0 : iGameMax+10 );
  }

  public Insets			getInsets()
  {
    return new Insets( 2, 4, 2, 4 );
  }

  public void			paint(
  Graphics			oGraphics )
  {
    Dimension			lDimension = getSize();

    setBackground( Color.white );
    oGraphics.drawRect( 1, 1, lDimension.width-2, lDimension.height-2 );
  }
}

class PanelHidden extends GridPanel
        implements ItemListener
{
  private Label			mLabelHidden;
  private Checkbox []		maCBoxHidden;
  private PbnVwHndlr		mPbnVwHndlr;

  public			PanelHidden(
  PbnVwHndlr			oPbnVwHndlr )
  {
    mPbnVwHndlr = oPbnVwHndlr;

    mLabelHidden = new Label( "Hidden Hands", Label.CENTER );
    maCBoxHidden = new Checkbox[ PbnSide.NUMBER ];

    /*
     * Left most side is WEST !
     */
    PbnSide			lSide = new PbnSide( PbnSide.WEST );

    for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
    {
      maCBoxHidden[ iSide ] = new Checkbox( lSide.toCharacter() + "     " );
      lSide.Next();
    }

    GridBagLayout		lGbl = new GridBagLayout();
    GridBagConstraints		lGbc = new GridBagConstraints();

    setLayout( lGbl );

    lGbc.anchor = GridBagConstraints.CENTER;
    lGbc.fill   = GridBagConstraints.BOTH;

    addGrid( mLabelHidden, lGbc, 0, 0, 4, 1 );
    for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
    {
      addGrid( maCBoxHidden[ iSide ], lGbc, iSide, 1, 1, 1 );
      maCBoxHidden[ iSide ].addItemListener( this );
    }
  }

    public void			itemStateChanged(
    ItemEvent			oEvent )
    {
      boolean []		baHidden = new boolean[ PbnSide.NUMBER ];
      PbnSide			lSide = new PbnSide( PbnSide.WEST );

      for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
      {
        baHidden[ lSide.Get() ] = maCBoxHidden[ iSide ].getState();
        lSide.Next();
      }

      mPbnVwHndlr.SetHidden( baHidden );
    }

  public void			Set(
  boolean []			baHiddenSide )
  {
    PbnSide			lSide = new PbnSide( PbnSide.WEST );

    for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
    {
      maCBoxHidden[ iSide ].setState( baHiddenSide[ lSide.Get() ] );
      lSide.Next();
    }
  }

  public Insets			getInsets()
  {
    return new Insets( 2, 4, 2, 4 );
  }

  public void			paint(
  Graphics			oGraphics )
  {
    Dimension			lDimension = getSize();

    setBackground( Color.white );
    oGraphics.drawRect( 1, 1, lDimension.width-2, lDimension.height-2 );
  }
}

class PanelControl extends Panel
{
  private static final int      NUMBER = 6;
  ControlButton []              maControlButtons;

  public                        PanelControl(
  PbnVwHndlr			oPbnVwHndlr )
  {
    // 1 row of panels
    GridLayout			lGl = new GridLayout( 1, 0, 2, 0 );
    setLayout( lGl );

    maControlButtons = new ControlButton[ NUMBER ];
    for ( int i = 0; i < NUMBER; i++ )
    {
      maControlButtons[ i ] = new ControlButton( oPbnVwHndlr, i );
      add( maControlButtons[ i ] );
    }
  }

  public Insets			getInsets()
  {
    return new Insets( 2, 8, 0, 4 );
  }
}

class ControlButton extends Canvas
        implements MouseListener
{
  private PbnVwHndlr            mPbnVwHndlr;
  private int                   mIndex;

  public                        ControlButton(
  PbnVwHndlr                    oPbnVwHndlr,
  int                           iIndex )
  {
    mPbnVwHndlr = oPbnVwHndlr;
    mIndex = iIndex;

    addMouseListener( this );
    setSize( 34, 34 );
  }

  public void                   mouseClicked(
  MouseEvent                    oEvent )
  {
  }

  public void                   mouseEntered(
  MouseEvent                    oEvent )
  {
  }

  public void                   mouseExited(
  MouseEvent                    oEvent )
  {
  }

  public void                   mousePressed(
  MouseEvent                    oEvent )
  {
    switch ( mIndex )
    {
    case 0:
      mPbnVwHndlr.PlayHead();
      break;
    case 1:
      mPbnVwHndlr.PlayBack();
      break;
    case 2:
      mPbnVwHndlr.PlayStop();
      break;
    case 3:
      mPbnVwHndlr.PlayExec();
      break;
    case 4:
      mPbnVwHndlr.PlayAuto();
      break;
    case 5:
      mPbnVwHndlr.PlayTail();
      break;
    }
  }

  public void                   mouseReleased(
  MouseEvent                    oEvent )
  {
  }

  public void			paint(
  Graphics			oGraphics )
  {
    Image mControlImage = MyImages.Get( mIndex );
    oGraphics.drawImage( mControlImage, 0, 0, this );
  }
}

class PanelFiles extends GridPanel
        implements ActionListener
{
  private PbnVwHndlr		mPbnVwHndlr;
  private Button            mFile1;

  public                    PanelFiles(
  PbnVwHndlr			oPbnVwHndlr )
  {
    mPbnVwHndlr = oPbnVwHndlr;

    mFile1 = addButton( "file1" );

    GridLayout			lGl = new GridLayout( 1, 0, 2, 2 );
    setLayout( lGl );

    add( mFile1 );
  }

  public Button                 addButton(
  String                        oName )
  {
    Button                      lButton = new Button( oName );

    lButton.addActionListener( this );

    return lButton;
  }

  public void                   actionPerformed(
  ActionEvent                   evt )
  {
    String                      lName = evt.getActionCommand();

    if ( lName.equals( "file1" ) )
    {
//      mPbnVwHndlr.ReadPbnURL( "test.pbn" );
    }
  }

  public void			paint(
  Graphics			oGraphics )
  {
    Dimension			lDimension = getSize();

    setBackground( Color.white );
    oGraphics.drawRect( 1, 1, lDimension.width-2, lDimension.height-2 );
  }
}

