/*
 * File   :     PanelTable.java
 * Author :     Tis Veugen
 * Date   :     2004-08-23
 * PBN    :     1.0
 *
 * History
 * -------
 * 2004-08-23 Added Rotate()
 */

import java.awt.*;

public class PanelTable extends GridPanel
{
  static public final Color	ColorBG = new Color( 128, 64, 0 );
  private PanelBoard		mPanelBoard;
  private PanelTricks		mPanelTricks;
  private PanelAuction		mPanelAuction;
  private PanelResult		mPanelResult;
  private PanelPlay		mPanelPlay;
  private PanelPlayer []	maPlayers;
  private int                   mNrTricksNS;
  private int                   mNrTricksEW;

  public			PanelTable()
  {
    setBackground( ColorBG );
    setFont( PanelScreen.FontCards );

    mPanelBoard   = new PanelBoard();
    mPanelTricks  = new PanelTricks();
    mPanelAuction = new PanelAuction();
    mPanelResult  = new PanelResult();
    mPanelPlay    = new PanelPlay();
    maPlayers     = new PanelPlayer[ PbnSide.NUMBER ];
    for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
    {
      maPlayers[ iSide ] = new PanelPlayer();
    }

    GridBagLayout		lGbl = new GridBagLayout();
    GridBagConstraints		lGbc = new GridBagConstraints();

    setLayout( lGbl );

    lGbc.anchor = GridBagConstraints.CENTER;
    lGbc.fill   = GridBagConstraints.BOTH;

    addGrid( mPanelBoard,   lGbc, 0, 0, 1, 1 );
    addGrid( mPanelTricks,  lGbc, 2, 0, 1, 1 );
    addGrid( mPanelAuction, lGbc, 0, 2, 1, 1 );
    addGrid( mPanelResult,  lGbc, 2, 2, 1, 1 );
    addGrid( mPanelPlay,    lGbc, 1, 1, 1, 1 );

    addGrid( maPlayers[ PbnSide.NORTH ], lGbc, 1, 0, 1, 1 );
    addGrid( maPlayers[ PbnSide.WEST ],  lGbc, 0, 1, 1, 1 );
    addGrid( maPlayers[ PbnSide.EAST ],  lGbc, 2, 1, 1, 1 );
    addGrid( maPlayers[ PbnSide.SOUTH ], lGbc, 1, 2, 1, 1 );
  }

  public Insets			getInsets()
  {
    return new Insets( 2, 0, 2, 0 );
  }

  public void			SetBoard(
  String			oBoardNr )
  {
    mPanelBoard.SetBoard( oBoardNr );
  }

  public void			SetSituation(
  PbnSituation			oSituation )
  {
    mPanelBoard.SetSituation( oSituation );
  }

  public void			ClearContract()
  {
    mPanelAuction.ClearContract();
  }

  public void			SetContract(
  PbnContract			oContract )
  {
    mPanelAuction.SetContract( oContract );
  }

  public void			ClearResult()
  {
    mPanelResult.ClearResult();
  }

  public void			SetResult(
  PbnResult			oResult )
  {
    mPanelResult.SetResult( oResult );
  }

  public void			ClearNrTricks()
  {
    mPanelTricks.ClearNrTricks();
  }

  public void			SetNrTricks(
  int				iNrTricksNS,
  int				iNrTricksEW )
  {
    mNrTricksNS = iNrTricksNS;
    mNrTricksEW = iNrTricksEW;
    mPanelTricks.SetNrTricks( iNrTricksNS, iNrTricksEW );
  }

  public void			ClearOptNrTricks()
  {
    mPanelResult.ClearOptNrTricks();
  }

  public void			SetOptNrTricks(
  PbnSide                       oSide,
  int				iNrTricks )
  {
    if ( oSide.IsNS() )
    {
      mPanelResult.SetOptNrTricks( "NS", mNrTricksNS, iNrTricks );
    }
    else
    {
      mPanelResult.SetOptNrTricks( "EW", mNrTricksEW, iNrTricks );
    }
  }

  public void			SetHidden(
  boolean []			abHiddenSide )
  {
    for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
    {
      maPlayers[ iSide ].SetHidden( abHiddenSide[ iSide ] );
    }
  }

  public void			PlayCard(
  PbnPlayCard                   oPlayCard,
  char                          cIllegal,
  int                           iNote )
  {
    mPanelPlay.PlayCard( oPlayCard, cIllegal, iNote );
  }

  public void			PlayClearAll()
  {
    mPanelPlay.PlayClearAll();
  }

  public void			PlayClearSide(
  PbnSide			oSide )
  {
    mPanelPlay.PlayClearSide( oSide );
  }

  public void			PlayQuestion(
  PbnSide			oSide )
  {
    mPanelPlay.PlayQuestion( oSide );
  }

  public void			SetName(
  PbnSide			oSide,
  String			oName )
  {
    maPlayers[ oSide.Get() ].SetName( oName );
  }

  public void			ClearHand(
  PbnSide			oSide )
  {
    maPlayers[ oSide.Get() ].ClearHand();
  }

  public void			SetHand(
  PbnSide			oSide,
  PbnHand			oHand )
  {
    maPlayers[ oSide.Get() ].SetHand( oHand );
  }

  public void			SetHand(
  PbnSide			oSide,
  PbnHand			oHand,
  PbnHand			oOptHand )
  {
    maPlayers[ oSide.Get() ].SetHand( oHand, oOptHand );
  }

  public void                   Rotate(
  int                           number )
  {
    mPanelPlay.Rotate( number );
  }
}

class PanelBoard extends GridPanel
{
  private TextLabel             mLabelBoard;
  private TextLabel             mLabelEmpty;
  private TextLabel             mLabelSituation;

  public			PanelBoard()
  {
    mLabelBoard     = new TextLabel();
    mLabelEmpty     = new TextLabel();
    mLabelSituation = new TextLabel();

    // 1 column of panels
    GridLayout			lGl = new GridLayout( 3, 1, 0, 0 );
    setLayout( lGl );

    add( mLabelBoard );
    add( mLabelEmpty );
    add( mLabelSituation );
  }

  public Insets			getInsets()
  {
    return new Insets( 2, 8, 2, 8 );
  }

  public void			SetBoard(
  String			oBoardNr )
  {
    if ( oBoardNr == null )
    {
      oBoardNr = "";
    }
    mLabelBoard.Draw( "Board: " + oBoardNr );
  }

  public void			SetSituation(
  PbnSituation			oSituation )
  {
    String                      lString = "";

    if ( oSituation != null )
    {
      lString = oSituation.GetDealer().toCharacter()
              + " / "
              + oSituation.GetVulner().toString();
    }
    mLabelSituation.Draw( lString );
  }
}

class PanelTricks extends Panel
{
  private TextLabel		mLabelNS;
  private TextLabel		mLabelEW;
  private TextLabel		mLabelEmpty;

  public			PanelTricks()
  {
    mLabelNS    = new TextLabel();
    mLabelEW    = new TextLabel();
    mLabelEmpty = new TextLabel();

    // 1 column of panels
    GridLayout			lGl = new GridLayout( 3, 1, 0, 0 );
    setLayout( lGl );

    add( mLabelNS );
    add( mLabelEW );
    add( mLabelEmpty );
  }

  public Insets			getInsets()
  {
    return new Insets( 2, 8, 2, 8 );
  }

  public void			ClearNrTricks()
  {
    mLabelNS.Clear();
    mLabelEW.Clear();
  }

  public void			SetNrTricks(
  int				iNrTricksNS,
  int				iNrTricksEW )
  {
    mLabelNS.Draw( "NS : " + iNrTricksNS );
    mLabelEW.Draw( "EW : " + iNrTricksEW );
  }
}

class PanelAuction extends Panel
{
  private TextLabel		mLabelEmpty1;
  private CanvasContract        mCanvasContract;
  private TextLabel		mLabelEmpty2;

  public			PanelAuction()
  {
    mLabelEmpty1    = new TextLabel();
    mCanvasContract = new CanvasContract();
    mLabelEmpty2    = new TextLabel();

    // 1 column of panels
    GridLayout			lGl = new GridLayout( 3, 1, 0, 0 );
    setLayout( lGl );

    add( mLabelEmpty1 );
    add( mCanvasContract );
    add( mLabelEmpty2 );
  }

  public Insets			getInsets()
  {
    return new Insets( 2, 8, 2, 8 );
  }

  public void			ClearContract()
  {
    mCanvasContract.ClearContract();
  }

  public void			SetContract(
  PbnContract			oContract )
  {
    mCanvasContract.SetContract( oContract );
  }
}

class CanvasContract extends Canvas
{
  private static final int	HEIGHT = 16;

  private boolean               mbContract;
  private PbnContract		mContract;

  public                        CanvasContract()
  {
    mbContract = false;
  }

  public void			paint(
  Graphics			oGraphics )
  {
    Dimension                   lDimension = getSize();
    int                         lW = lDimension.width;
    int                         lH = lDimension.height;

    if ( ! mbContract )
    {
      oGraphics.setColor( PanelTable.ColorBG );
      oGraphics.fillRect( 0, 0, lW, lH );
    }
    else
    {
      String                    lString = " ";
      int                       x = 0;
      int                       y = (lH - HEIGHT)/2;
      int                       w, h;
      FontMetrics               lFontMetrics = oGraphics.getFontMetrics();

      oGraphics.setColor( Color.white );
      oGraphics.fillRect( 0, 0, lW, lH );

      oGraphics.setColor( Color.black );
      if ( mContract.GetIrregularDeclarer() )
      {
        lString = lString + "^";
      }

      lString = lString
              + mContract.GetDeclarer().toCharacter()
              + ": ";
      if ( mContract.GetNrTricks() != 0 )
      {
        lString = lString + mContract.GetNrTricks();
      }
      oGraphics.drawString( lString, x, y+HEIGHT-3 );

      w = lFontMetrics.stringWidth( lString );
      x = w+2;

      w = 0;
      Image mTrumpImage = MyImages.Get( mContract.GetTrump() );
      if ( mTrumpImage != null )
      {
        w = mTrumpImage.getWidth( this );
        oGraphics.drawImage( mTrumpImage, x, y, this );
      }

      x += w+2;
      lString = mContract.GetRisk().toString();
      oGraphics.drawString( lString, x, y+HEIGHT-3 );
    }
  }

  public void			ClearContract()
  {
    mbContract = false;
    repaint();
  }

  public void			SetContract(
  PbnContract			oContract )
  {
    mbContract = true;
    mContract = oContract;
    repaint();
  }
}

class PanelResult extends Panel
{
  private TextLabel		mLabelTricks;
  private TextLabel		mLabelEmpty;
  private TextLabel		mLabelResult;

  public			PanelResult()
  {
    mLabelTricks = new TextLabel();
    mLabelEmpty  = new TextLabel();
    mLabelResult = new TextLabel();

    // 1 column of panels
    GridLayout			lGl = new GridLayout( 3, 1, 0, 0 );
    setLayout( lGl );

    add( mLabelTricks );
    add( mLabelEmpty  );
    add( mLabelResult );
  }

  public Insets			getInsets()
  {
    return new Insets( 2, 8, 2, 8 );
  }

  public void			ClearResult()
  {
    mLabelResult.Clear();
    mLabelTricks.Clear();
  }

  public void			ClearOptNrTricks()
  {
    mLabelTricks.Clear();
  }

  public void			SetOptNrTricks(
  String                        oSides,
  int                           iNrCurrentTricks,
  int                           iNrRestTricks )
  {
    mLabelTricks.Draw( oSides + ": " + iNrCurrentTricks + " + " + iNrRestTricks );
  }

  public void			SetResult(
  PbnResult			oResult )
  {
    mLabelResult.Draw( "Result: " + oResult.toString() );
  }
}

class PanelPlay extends Panel
{
  private static final int	WIDTH  = 50;
  private static final int	HEIGHT = 16;
  private FieldPlay []          maFieldPlays;
  private boolean               mbShow = false;
  private int                   mRotate = 0;

  public			PanelPlay()
  {
    maFieldPlays = new FieldPlay[ PbnSide.NUMBER ];
    for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
    {
      maFieldPlays[ iSide ] = new FieldPlay();
    }
  }

  public void			paint(
  Graphics			oGraphics )
  {
    Dimension			lDimension = getSize();
    int                         x, y;

    x = lDimension.width/2 - WIDTH/2;
    y = lDimension.height/10;
    maFieldPlays[ PbnSide.NORTH ].Render( this, x, y, oGraphics );

    y = lDimension.height-1 - lDimension.height/10 - HEIGHT;
    maFieldPlays[ PbnSide.SOUTH ].Render( this, x, y, oGraphics );

    x = lDimension.width/10;
    y = lDimension.height/2 - HEIGHT/2;
    maFieldPlays[ PbnSide.WEST ].Render( this, x, y, oGraphics );

    x = lDimension.width-1 - lDimension.width/10 - WIDTH;
    maFieldPlays[ PbnSide.EAST ].Render( this, x, y, oGraphics );

    if ( mbShow && (mRotate != 0) )
    {
      String                    s = "Rotated x " + mRotate;
      FontMetrics               lFontMetrics = oGraphics.getFontMetrics();
      int                       w = lFontMetrics.stringWidth( s );
      int                       h = lFontMetrics.getHeight();

      x = lDimension.width/2 - w/2;
      y = lDimension.height/2;

      oGraphics.setColor( Color.white );
      oGraphics.fillRect( x-4, y-h, 4+w+4, h+4 );

      oGraphics.setColor( new Color( 0, 100, 200 ) );
      oGraphics.drawString( s, x, y );

      oGraphics.setColor( PanelTable.ColorBG );
    }
  }

  public void			PlayCard(
  PbnPlayCard                   oPlayCard,
  char                          cIllegal,
  int                           iNote )
  {
    mbShow = false;
    maFieldPlays[ oPlayCard.GetSide().Get() ].Draw( oPlayCard.GetCard()
                                                  , cIllegal
                                                  , iNote );
    repaint();
  }

  public void			PlayClearAll()
  {
    mbShow = true;
    for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
    {
      maFieldPlays[ iSide ].Clear();
    }
    repaint();
  }

  public void			PlayClearSide(
  PbnSide			oSide )
  {
    maFieldPlays[ oSide.Get() ].Clear();
    repaint();
  }

  public void			PlayQuestion(
  PbnSide			oSide )
  {
    mbShow = false;
    maFieldPlays[ oSide.Get() ].Question();
    repaint();
  }

  public void                   Rotate(
  int                           number )
  {
    mRotate = number;
  }
}

class FieldPlay
{
  private static final int	WIDTH  = 50;
  private static final int	HEIGHT = 16;

  private static final int	CLEAR    = 0;
  private static final int	QUESTION = 1;
  private static final int	CARD     = 2;

  private int                   mWhat = CLEAR;
  private PbnCard               mCard;
  private char                  mcIllegal;
  private int                   mNote;

  public void			Render(
  Panel                         oPanel,
  int                           x,
  int                           y,
  Graphics			oGraphics )
  {
    FontMetrics                 lFontMetrics = oGraphics.getFontMetrics();
    int                         w;
    String                      lString;

    switch ( mWhat )
    {
    case CLEAR:
      oGraphics.setColor( PanelTable.ColorBG );
      oGraphics.fillRect( x, y, WIDTH, HEIGHT );
      break;

    case QUESTION:
      oGraphics.setColor( Color.white );
      oGraphics.fillRect( x, y, WIDTH, HEIGHT );

      oGraphics.setColor( Color.black );
      lString = "?";
      w = lFontMetrics.stringWidth( lString );
      oGraphics.drawString( lString, x + (WIDTH-w)/2, y + HEIGHT-3 );
      break;

    case CARD:
      oGraphics.setColor( Color.white );
      oGraphics.fillRect( x, y, WIDTH, HEIGHT );

      oGraphics.setColor( Color.black );
      if ( ! ( mCard.GetSuit().IsValid()
            && mCard.GetRank().IsNormal() ) )
      {
        lString = "-";
        w = lFontMetrics.stringWidth( lString );
        oGraphics.drawString( lString, x + (WIDTH-w)/2, y + HEIGHT-3 );
      }
      else
      {
        w = 0;
        if ( mcIllegal != '\0' )
        {
          lString = "^" + mcIllegal;
          w = lFontMetrics.stringWidth( lString ) + 2;
        }

        Image mSuitImage = MyImages.Get( mCard.GetSuit() );
        if ( mSuitImage != null )
        {
          w += mSuitImage.getWidth( oPanel ) + 2;
        }

        lString = mCard.GetRank().toCharacter();
        w += lFontMetrics.stringWidth( lString );

        // Compute 'x' and draw it.
        int X = x + (WIDTH - w) / 2;

        if ( mcIllegal != '\0' )
        {
          lString = "^" + mcIllegal;
          oGraphics.setColor( PanelScreen.ColorPink );
          oGraphics.drawString( lString, X, y + HEIGHT-3 );
          X += lFontMetrics.stringWidth( lString ) + 2;
          oGraphics.setColor( Color.black );
        }

        if ( mSuitImage != null )
        {
          oGraphics.drawImage( mSuitImage, X, y, oPanel );
          X += mSuitImage.getWidth( oPanel ) + 2;
        }

        lString = mCard.GetRank().toCharacter();
        oGraphics.drawString( lString, X, y + HEIGHT-3 );

        if ( mNote != 0 )
        {
          oGraphics.setFont( PanelScreen.FontNote );
          oGraphics.setColor( PanelScreen.ColorDarkGreen );
          lString = "" + mNote;
          w = lFontMetrics.stringWidth( lString );
          oGraphics.drawString( lString, x + WIDTH-1 - w, y + 12 );
          // Restore font
          oGraphics.setFont( PanelScreen.FontCards );
        }
      }
      break;
    }
  }

  public void			Clear()
  {
    mWhat = CLEAR;
  }

  public void			Draw(
  PbnCard                       oCard,
  char                          cIllegal,
  int                           iNote )
  {
    mWhat = CARD;
    mCard = new PbnCard( oCard );
    mcIllegal = cIllegal;
    mNote = iNote;
  }

  public void			Question()
  {
    mWhat = QUESTION;
  }
}

class PanelPlayer extends GridPanel
{
  private PanelCards []		maPanelCards;
  private LabelName		mLabelName;
  private boolean               mbHidden;
  private int []                maPeerRanks =
  {
    PbnSuit.DIAMONDS,
    PbnSuit.CLUBS,
    PbnSuit.DIAMONDS,
    PbnSuit.HEARTS
  };

  public			PanelPlayer()
  {
    PbnSuit                     lSuit = new PbnSuit( PbnSuit.SPADES );

    maPanelCards = new PanelCards[ PbnSuit.NUMBER ];
    for ( int iSuit = 0; iSuit < PbnSuit.NUMBER; iSuit++ )
    {
      maPanelCards[ iSuit ] = new PanelCards( lSuit );
      lSuit.Previous();
    }
    mLabelName = new LabelName();

    GridBagLayout		lGbl = new GridBagLayout();
    GridBagConstraints		lGbc = new GridBagConstraints();

    setLayout( lGbl );

    lGbc.anchor = GridBagConstraints.CENTER;
    lGbc.fill   = GridBagConstraints.BOTH;

    for ( int iSuit = 0; iSuit < PbnSuit.NUMBER; iSuit++ )
    {
      addGrid( maPanelCards[ iSuit ], lGbc
             , 0, iSuit, 1, 1 );
    }
    addGrid( mLabelName, lGbc, 0, PbnSuit.NUMBER, 1, 1 );
  }

  public Insets			getInsets()
  {
    return new Insets( 1, 1, 0, 0 );
  }

  public void			paint(
  Graphics			oGraphics )
  {
    Dimension			lDimension = getSize();

    setBackground( Color.black );
    oGraphics.drawRect( 0, 0, lDimension.width-1, lDimension.height-1 );
  }

  public void			update(
  Graphics			oGraphics )
  {
    paint( oGraphics );
  }

  public void			SetName(
  String			oName )
  {
    mLabelName.SetName( oName );
  }

  public void			SetHidden(
  boolean                       bHidden )
  {
    mbHidden = bHidden;
  }

  public void			ClearHand()
  {
    for ( int iSuit = 0; iSuit < PbnSuit.NUMBER; iSuit++ )
    {
      maPanelCards[ iSuit ].ClearRanks();
    }
  }

  public void			SetHand(
  PbnHand			oHand )
  {
    SetHand( oHand, new PbnHand() );
  }

  private String                FilterRanks(
  String                        oRanks,
  String                        oOptRanks )
  {
    int                         n = oRanks.length();
    StringBuffer                sb = new StringBuffer();

    for ( int i = 0; i < n; i++ )
    {
      if ( oOptRanks.indexOf( oRanks.substring( i, i+1 ) ) >= 0 )
      {
        sb.append( "O" );
      }
      else
      {
        sb.append( " " );
      }
    }
    return sb.toString();
  }

  public void			SetHand(
  PbnHand			oHand,
  PbnHand			oOptHand )
  {
    String []                   laRanks = new String[ PbnSuit.NUMBER ];
    String []                   laOptRanks = new String[ PbnSuit.NUMBER ];
    String                      lString;

    ClearHand();

    if ( mbHidden )
    {
      return;
    }

    PbnSuit			lSuit = new PbnSuit( PbnSuit.CLUBS );

    for ( int iSuit = 0; iSuit < PbnSuit.NUMBER; iSuit++ )
    {
      laRanks[ iSuit ] = oHand.GetRanks( lSuit ).toString();
      lString = oOptHand.GetRanks( lSuit ).toString();
      if ( lString.equals( "" ) )
      {
        laOptRanks[ iSuit ] = PanelCards.NO_OPT_CARDS;
      }
      else
      {
        laOptRanks[ iSuit ] = FilterRanks( laRanks[ iSuit ]
                                         , lString );
      }
      lSuit.Next();
    }

    maPanelCards[ 3 ].SetRanks( laRanks[ PbnSuit.CLUBS ]
                              , laRanks[ PbnSuit.DIAMONDS ]
                              , laOptRanks[ PbnSuit.CLUBS ]
                              , laOptRanks[ PbnSuit.DIAMONDS ] );
    int                 lPeerSuit;
    if ( laRanks[ PbnSuit.CLUBS ].length() > PanelCards.NR_CARDS )
    {
      lPeerSuit = PbnSuit.CLUBS;
    }
    else
    {
      lPeerSuit = PbnSuit.HEARTS;
    }
    maPanelCards[ 2 ].SetRanks( laRanks[ PbnSuit.DIAMONDS ]
                              , laRanks[ lPeerSuit ]
                              , laOptRanks[ PbnSuit.DIAMONDS ]
                              , laOptRanks[ lPeerSuit ] );
    maPanelCards[ 1 ].SetRanks( laRanks[ PbnSuit.HEARTS ]
                              , laRanks[ PbnSuit.SPADES ]
                              , laOptRanks[ PbnSuit.HEARTS ]
                              , laOptRanks[ PbnSuit.SPADES ] );
    maPanelCards[ 0 ].SetRanks( laRanks[ PbnSuit.SPADES ]
                              , laOptRanks[ PbnSuit.SPADES ] );
  }
}

class PanelCards extends Canvas
{
  public static final int	NR_CARDS = 9;
  public static final String    NO_OPT_CARDS = "             ";
  private static final int	HEIGHT = 18;
  private static final int	W_IMAGE = 18;
  private static final int	W_CARDS = 12;

  private Image                 mSuitImage;
  private String                mSuitCards = "";
  private String                mNoCards = "         ";
  private String                mOptCards = NO_OPT_CARDS;

  public			PanelCards(
  PbnSuit                       oSuit )
  {
    mSuitImage = MyImages.Get( oSuit );

    setFont( PanelScreen.FontCards );
    setSize( W_IMAGE + 4 + NR_CARDS * W_CARDS, HEIGHT );
  }

  public void			ClearRanks()
  {
    mSuitCards = mNoCards;
    mOptCards  = mNoCards;
    repaint();
  }

  public void			SetRanks(
  String   			oCards,
  String   			oOptCards )
  {
    int                         n = oCards.length();
    if ( n >= NR_CARDS )
    {
      mSuitCards = oCards.substring( 0, NR_CARDS );
      mOptCards  = oOptCards.substring( 0, NR_CARDS );
    }
    else
    {
      mSuitCards = oCards + mNoCards.substring( 0, NR_CARDS-n );
      mOptCards  = oOptCards + mNoCards.substring( 0, NR_CARDS-n );
    }

    repaint();
  }

  public void			SetRanks(
  String   			oCards,
  String   			oPeerCards,
  String   			oOptCards,
  String   			oOptPeerCards )
  {
    int                         n = oCards.length();
    if ( n >= NR_CARDS )
    {
      mSuitCards = oCards.substring( 0, NR_CARDS );
      mOptCards  = oOptCards.substring( 0, NR_CARDS );
    }
    else
    {
      mSuitCards = oCards + mNoCards.substring( 0, NR_CARDS-n );
      mOptCards  = oOptCards + mNoCards.substring( 0, NR_CARDS-n );
      n = oPeerCards.length();
      if ( n > NR_CARDS )
      {
        mSuitCards = mSuitCards.substring( 0, NR_CARDS - (n-NR_CARDS) )
                   + oPeerCards.substring( NR_CARDS, n );
        mOptCards = mOptCards.substring( 0, NR_CARDS - (n-NR_CARDS) )
                  + oOptPeerCards.substring( NR_CARDS, n );
      }
    }

    repaint();
  }

  public void			paint(
  Graphics			oGraphics )
  {
    oGraphics.setColor( Color.green );
    oGraphics.fillRect( 0, 0, W_IMAGE + 4 + NR_CARDS * W_CARDS, HEIGHT );
    oGraphics.drawImage( mSuitImage, 1, 1, this );

    int                 x = W_IMAGE + 4;
    int                 n = mSuitCards.length();

    oGraphics.setColor( Color.black );
    for ( int i = 0; i < n; i++ )
    {
      if ( mOptCards.charAt( i ) != ' ' )
      {
        oGraphics.setColor( Color.yellow );
        oGraphics.fillRect( x, 1, W_CARDS, HEIGHT-2 );
        oGraphics.setColor( Color.black );
      }
      oGraphics.drawString( "" + mSuitCards.charAt( i )
                          , x, HEIGHT-3 );
      x += W_CARDS;
    }
  }
}

class LabelName extends Label
{
  public                        LabelName()
  {
    setAlignment( Label.CENTER );
    setBackground( Color.white );
    setFont( PanelScreen.FontNames );
  }

  public void			SetName(
  String			oName )
  {
    if ( oName == null )
    {
      oName = "";
    }
    setForeground( Color.black );
    setText( oName );
  }

  public Insets			getInsets()
  {
    return new Insets( 1, 0, 0, 0 );
  }

  public void			paint(
  Graphics			oGraphics )
  {
    Dimension			lDimension = getSize();

    oGraphics.setColor( Color.black );
    oGraphics.drawLine( 0, 0, lDimension.width-1, 0 );
  }
}

class TextLabel extends Label
{
  public                        TextLabel()
  {
    setForeground( Color.black );
    Clear();
  }

  public void			Clear()
  {
    setBackground( PanelTable.ColorBG );
    setText( "" );
  }

  public void			Draw(
  String                        oString )
  {
    setBackground( Color.white );
    setText( " " + oString );
  }
}
