/*
 * File   :     PbnFile.java
 * Author :     Tis Veugen
 * Date   :     1999-10-23
 * PBN    :     2.0
 *
 * History
 * -------
 * 1999-06-12 Check severity for accepting a game
 * 1999-10-10 Cleanup
 * 1999-10-23 Applet
 */

import java.io.*;
import java.util.*;

class PbnGameEntry
{
  public long                   mFilePosIn;
  public long                   mFilePosOut;
  public PbnUseTagIds           mUseTagIds;

  public                        PbnGameEntry(
  long                          iFilePosIn,
  long                          iFilePosOut,
  PbnUseTagIds                  oUseTagIds )
  {
    mFilePosIn  = iFilePosIn;
    mFilePosOut = iFilePosOut;
    mUseTagIds  = oUseTagIds;
  }
}

public class PbnFile
{
  private PbnImport             mImport;
  private Integer []            maValidGames;
  private PbnGameEntry []       maGameEntrys;

  public                        PbnFile(
  PbnInputStream                oInputStream,
  FileOutputStream              oLoggingFos )
  {
    mImport = new PbnImport();

    mImport.SetInputFile( oInputStream );
    mImport.SetLogFile( oLoggingFos );
  }

  public void                   Read()
  {
    long                        FilePosIn = 0;
    long                        FilePosOut;
    int                         GameIndex = 0;
    PbnInherit                  lInherit = new PbnInherit();
    Vector                      lvGameEntrys = new Vector();
    Vector                      lvValidGames = new Vector();
    PbnError                    lError;

    mImport.SetInherit( lInherit );

    PbnGen.SetParsing( PbnGen.PARSE_FIRST );

    do
    {
      PbnGame                   lGame = new PbnGame();
      PbnUseTagIds              lUseTagIds = new PbnUseTagIds();

      /*
       * Put the '##' tags already in the game's GameTags.
       */
      lInherit.GetCopyTags( lGame.GetGameTags() );
      /*
       * Store the game numbers of the '##' tags in the game's UseTagIds.
       */
      lInherit.GetCopyGameNrs( lUseTagIds );
      lInherit.SetGame( GameIndex );
      lInherit.ClearPrevGameNrs();

      mImport.LogSetGameNr( GameIndex + 1 );
      lError = mImport.Read( FilePosIn
                           , lGame.GetGameData()
                           , lGame.GetGameTags() );

      FilePosOut = mImport.mFilePosOut;

      if ( ( ! lError.Is( PbnError.NO_TAG )              )
        && ( ! lError.HasSeverity( PbnError.SEV_SEVERE ) ) )
      { /*
         * Game is playable when the error is less than SEV_SEVERE.
         */
        lvValidGames.addElement( new Integer( GameIndex ) );
        /*
         * Store the game numbers of the '#' tags in the game's UseTagIds.
         */
        lInherit.GetPrevGameNrs( lUseTagIds );
      }

      PbnGameEntry            lGameEntry;

      lGameEntry = new PbnGameEntry( FilePosIn
                                   , FilePosOut
                                   , lUseTagIds );
      lvGameEntrys.addElement( lGameEntry );

      GameIndex++;
      FilePosIn = FilePosOut;
    } while ( ! mImport.IsEof() );

    mImport.CloseLogFile();

    /*
     * Copy the vectors into an array.
     */
    maValidGames = new Integer[ lvValidGames.size() ];
    lvValidGames.copyInto( maValidGames );
  
    maGameEntrys = new PbnGameEntry[ lvGameEntrys.size() ];
    lvGameEntrys.copyInto( maGameEntrys );
  }

  public int                    NrGames()
  {
    return maValidGames.length;
  }

  PbnGame                       GetGame(
  int                           iGameIndex )
  {
    int                         iGameNr = maValidGames[iGameIndex].intValue();
    PbnGameEntry                lGameEntry = maGameEntrys[ iGameNr ];

    PbnGame                     lGame = new PbnGame();
    PbnGameTags                 lGameTags = lGame.GetGameTags();

    PbnUseTagIds                lUseTagIds = lGameEntry.mUseTagIds;
    lUseTagIds.Reset();

    int                         iTagId;
    PbnError                    lError;

    PbnGen.SetParsing( PbnGen.PARSE_COPY );
    while ( (iGameNr = lUseTagIds.GetGameNr()) >= 0 )
    {
      PbnGameEntry              lCopyGameEntry = maGameEntrys[ iGameNr ];
      PbnGame                   lCopyGame = new PbnGame();
      PbnGameTags               lCopyGameTags = lCopyGame.GetGameTags();

      lError = mImport.Read( lCopyGameEntry.mFilePosIn
                           , lCopyGame.GetGameData()
                           , lCopyGameTags );

      /*
       * Copy all tags of lCopyGame to lGame,
       * provided that lUseTagIds points to iGameNr.
       */
      while ( (iTagId = lUseTagIds.GetTagId()) >= 0 )
      {
        lGameTags.InheritCopy( lCopyGameTags
                             , new PbnTagId( iTagId ) );
      }
    }

    PbnGen.SetParsing( PbnGen.PARSE_NEXT );
    lError = mImport.Read( lGameEntry.mFilePosIn
                         , lGame.GetGameData()
                         , lGameTags );

    String []                   laStrings;

    laStrings = mImport.ReadText( lGameEntry.mFilePosIn
                                , lGameEntry.mFilePosOut );
    lGame.SetGameTexts( laStrings );

    return lGame;
  }
}
