/*
 * File   :     PbnGame.java
 * Author :     Tis Veugen
 * Date   :     2004-08-23
 * PBN    :     2.0
 *
 * History
 * -------
 * 2002-02-22 Added table display
 * 2003-03-29 Take unknown cards into account
 * 2004-08-23 Added Rotate()
 */

import java.awt.*;

public class PbnGame
{
  private final int             NR_PLAY_CARDS = PbnTrick.NUMBER * PbnSide.NUMBER;

  private PbnGameData           mGameData;
  private PbnGameTags           mGameTags;
  private String []             maGameTexts;
  private PbnHand []            maOptHands;
  private int []                maOptNrTricks;

  public			PbnGame()
  {
    mGameData = new PbnGameData();
    mGameTags = new PbnGameTags();
    maOptHands    = new PbnHand[ NR_PLAY_CARDS ];
    maOptNrTricks = new int[ NR_PLAY_CARDS ];
  }

  public PbnGameData            GetGameData()
  {
    return mGameData;
  }

  public PbnGameTags            GetGameTags()
  {
    return mGameTags;
  }

  public String []              GetGameTexts()
  {
    return maGameTexts;
  }

  public void                   SetGameTexts(
  String []                     laStrings )
  {
    maGameTexts = laStrings;
  }

  public AnnoArea               GetAnnoArea()
  {
    return mAnnoArea;
  }

  private PanelScreen           mPanelScreen;
  private int                   mSection;
  private PbnDeal               mDeal;
  private PbnPlayAdmin          mPlayAdmin;
  private int                   mCallIndex;
  private int                   mCardIndex;
  private boolean               mbTrick;
  private AnnoArea              mAnnoArea;

  public void                   New(
  PanelScreen                   oPanelScreen )
  {
    PbnTagId                    lTagId = new PbnTagId();

    mPanelScreen = oPanelScreen;
    mDeal = new PbnDeal( mGameData.GetDeal() );

    mPlayAdmin = new PbnPlayAdmin();
    mPlayAdmin.Compute( mGameData );

    /*
     * Draw the fixed fields.
     */
    lTagId.Set( PbnTagId.BOARD );
    mPanelScreen.SetBoard( mGameTags.GetTagString( lTagId ) );

    mPanelScreen.SetSituation( mGameData.GetSituation() );
    DrawNames( mGameTags );

    boolean []                  abHiddenSides;
    abHiddenSides = mGameTags.GetHiddenSides();
    mPanelScreen.SetHidden( abHiddenSides );
    mPanelScreen.TableHidden( abHiddenSides );
    DrawHands();
    /*
     * Draw the annotation area.
     */
    mPanelScreen.SetGameTexts( maGameTexts );

    mAnnoArea = new AnnoArea( oPanelScreen
                            , mPanelScreen.GetAreaWidth() );
    Image lAnnoImage = mAnnoArea.ComputeImage( mGameData
                                             , mGameTags
                                             , mPlayAdmin );
    mPanelScreen.SetAnnoImage( lAnnoImage );

    String [] laTableTexts = ComputeTables( mGameData
                                          , mGameTags );
    mPanelScreen.SetTableTexts( laTableTexts );

    SetSection( PbnGen.SECTION_IDENT );
  }

  private String []             ComputeTables(
  PbnGameData                   oGameData,
  PbnGameTags                   oGameTags )
  {
    String []                   laS1 = null;
    String []                   laS2 = null;
    int                         n = 0;

    for ( int tid = PbnTagId.ACTIONTABLE; tid <= PbnTagId.TOTALSCORETABLE; tid++ )
    {
      PbnTagId                  lTagId = new PbnTagId( tid );
      PbnTable                  lTable = oGameTags.GetTable( lTagId );

      if ( lTable.GetHeader() != null )
      {
        String [] lasT = lTable.toStrings( lTagId );

        /*
         * Copy the strings to 1 array.
         */
        int nt = lasT.length;
        laS2 = new String[ n + nt ];
        for ( int i = 0; i < n; i++ ) laS2[i] = laS1[i];
        for ( int i = 0; i < nt; i++ ) laS2[n+i] = lasT[i];
        laS1 = laS2;
        n += nt;
      }
    }

    if ( laS1 == null )
    {
      laS1 = new String[ 1 ];
      laS1[0] = "No tables";
    }
    return laS1;
  }

  public void                   Reset()
  {
    mCallIndex = 0;
    mCardIndex = 0;
    mbTrick = false;

    mDeal = new PbnDeal( mGameData.GetDeal() );
    DrawHands();

    mPanelScreen.ClearContract();
    mPanelScreen.ClearResult();
    mPanelScreen.ClearNrTricks();
    mPanelScreen.ClearOptNrTricks();
    mPanelScreen.HistClearAll();
    mPanelScreen.PlayClearAll();
  }

  private void                  DrawHands()
  {
    PbnSide                     lSide = new PbnSide( PbnSide.SOUTH );

    for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
    {
      mPanelScreen.SetHand( lSide, mDeal.GetHand( lSide ) );
      lSide.Next();
    }
  }

  private void                  DrawNames(
  PbnGameTags                   oGameTags )
  {
    PbnTagId                    lTagId = new PbnTagId();
    PbnSide                     lSide = new PbnSide( PbnSide.SOUTH );

    lTagId.Set( PbnTagId.SOUTH );
    mPanelScreen.SetName( lSide, oGameTags.GetTagString( lTagId ) );
    lSide.Next();
    lTagId.Set( PbnTagId.WEST );
    mPanelScreen.SetName( lSide, oGameTags.GetTagString( lTagId ) );
    lSide.Next();
    lTagId.Set( PbnTagId.NORTH );
    mPanelScreen.SetName( lSide, oGameTags.GetTagString( lTagId ) );
    lSide.Next();
    lTagId.Set( PbnTagId.EAST );
    mPanelScreen.SetName( lSide, oGameTags.GetTagString( lTagId ) );
  }

  public void			        SetSection(
  int				            iSection )
  {
    mSection = iSection;
    mPanelScreen.SetSection( mSection );
    PlayHead();
  }

  public void			        SetHidden(
  boolean []			        abHiddenSides )
  {
    mPanelScreen.TableHidden( abHiddenSides );
    DrawHands();
  }

  private boolean               AuctionIsDone()
  {
    return mCallIndex >= mGameData.GetAuction().GetNrCalls();
  }

  private boolean               PlayIsDone()
  {
    return mPlayAdmin.PlayIsDone( mCardIndex );
  }

  private void                  HandlePlayDone()
  {
    mPanelScreen.SetResult( mGameData.GetResult() );
  }

  private void                  SetNrTricks()
  {
    int                         iCardIndex = mCardIndex;

    if ( mbTrick )
    { /*
       * Subtract 1 to get the tricks before finishing the trick.
       */
      iCardIndex--;
    }

    if ( iCardIndex < PbnSide.NUMBER )
    {
      mPanelScreen.SetNrTricks( 0, 0 );
    }
    else
    {
      int                       iNrTricks = iCardIndex/PbnSide.NUMBER;
      int                       iNrTricksNS = mPlayAdmin.GetNrWonNS( iNrTricks-1 );
      int                       iNrTricksEW = mPlayAdmin.GetNrWonEW( iNrTricks-1 );

      mPanelScreen.SetNrTricks( iNrTricksNS, iNrTricksEW );
    }
  }

  private int                   GetRowForCall(
  int                           iCallIndex )
  {
    PbnSide                     lDealer = mGameData.GetSituation().GetDealer();
    int                         iOffset = -1;

    if ( lDealer.Is( PbnSide.SOUTH ) )
    {
      iOffset += PbnSide.NUMBER;
    }
    return ( lDealer.Get() + iOffset + iCallIndex )/PbnSide.NUMBER;
  }

  private void                  RefreshAnnoArea()
  {
    mPanelScreen.SetAnnoRange( mAnnoArea.GetRange() );
  }

  public void                   PlayHead()
  {
    Reset();
    switch ( mSection )
    {
    case PbnGen.SECTION_IDENT:
      break;

    case PbnGen.SECTION_AUCTION:
      PbnAuction                lAuction = mGameData.GetAuction();
      if ( ! AuctionIsDone() )
      {
        PbnCall lCall = lAuction.GetCall( mCallIndex );

        mPanelScreen.HistQuestion( lCall.GetSide(), 0 );
      }
      break;

    case PbnGen.SECTION_PLAY:
      mPanelScreen.SetContract( mGameData.GetContract() );
      SetNrTricks();
      if ( PlayIsDone() )
      {
        HandlePlayDone();
      }
      else
      {
        PbnPlayCard lPlayCard = mPlayAdmin.GetPlayCard( mCardIndex );

        mPanelScreen.PlayQuestion( lPlayCard.GetSide() );
        mPanelScreen.HistBar( lPlayCard.GetSide(), 0 );
      }
      break;
    }

    mAnnoArea.ShowSection( mSection );
    RefreshAnnoArea();
  }

  public void                   PlayBack()
  {
    switch ( mSection )
    {
    case PbnGen.SECTION_IDENT:
      break;

    case PbnGen.SECTION_AUCTION:
      PbnAuction                lAuction = mGameData.GetAuction();
      PbnCall                   lCall;

      if ( mCallIndex == 0 )
      {
        SetSection( PbnGen.SECTION_IDENT );
      }
      else
      {
        if ( ! AuctionIsDone() )
        {
          lCall = lAuction.GetCall( mCallIndex );
          mPanelScreen.PlayClearSide( lCall.GetSide() );
          mPanelScreen.HistClear( lCall.GetSide()
                                , GetRowForCall( mCallIndex ) );
        }

        lCall = lAuction.GetCall( --mCallIndex );
        mPanelScreen.HistQuestion( lCall.GetSide()
                                  , GetRowForCall( mCallIndex ) );
      }
      break;

    case PbnGen.SECTION_PLAY:
      mPanelScreen.ClearOptNrTricks();
      if ( PlayIsDone() )
      {
        mPanelScreen.ClearResult();
      }

      if ( mCardIndex == 0 )
      {
        SetSection( PbnGen.SECTION_AUCTION );
        PlayTail();
      }
      else
      {
        boolean                 bUnplay = false;
        PbnPlayCard             lPlayCard;

        if ( (mCardIndex & 0x03) == 0 )
        {
          if ( ! mbTrick )
          { /*
             * Remove 4 cards from history, and
             * show them on the table.
             */
            mbTrick = true;
            int                 iIndex = mCardIndex - PbnSide.NUMBER;
            int                 iTrick = iIndex/PbnSide.NUMBER;
            char                lcIllegal;

            if ( ! PlayIsDone() )
            { /*
               * Clear the bar.
               */
              lPlayCard = mPlayAdmin.GetPlayCard( mCardIndex );
              mPanelScreen.HistClear( lPlayCard.GetSide(), iTrick+1 );
            }

            for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
            {
              lPlayCard = mPlayAdmin.GetPlayCard( iIndex + iSide );
              lcIllegal = mPlayAdmin.GetIllegal( iTrick
                                               , lPlayCard.GetSide() );
              mPanelScreen.PlayCard( lPlayCard
                                   , lcIllegal
                                   , 0 );
              mPanelScreen.HistClear( lPlayCard.GetSide()
                                    , iTrick );
            }

            lPlayCard = mPlayAdmin.GetPlayCard( iIndex );
            mPanelScreen.HistBar( lPlayCard.GetSide(), iTrick );

            SetNrTricks();
          }
          else
          { /*
             * Replace 4th card by question.
             */
            mbTrick = false;
            bUnplay = true;
          }
        }
        else
        {
          if ( ! PlayIsDone() )
          {
            lPlayCard = mPlayAdmin.GetPlayCard( mCardIndex );
            mPanelScreen.PlayClearSide( lPlayCard.GetSide() );
          }
          bUnplay = true;
        }

        if ( bUnplay )
        { /*
           * Hide the optimal cards.
           */
          if ( ! PlayIsDone() )
          {
            lPlayCard = mPlayAdmin.GetPlayCard( mCardIndex );
            PbnSide lSide1 = lPlayCard.GetSide();
            mPanelScreen.SetHand( lSide1, mDeal.GetHand( lSide1 ) );
          }

          lPlayCard = mPlayAdmin.GetPlayCard( --mCardIndex );
          PbnSide lSide = lPlayCard.GetSide();

          mDeal.UnplayCard( lSide, lPlayCard.GetCard() );
          mPanelScreen.SetHand( lSide, mDeal.GetHand( lSide ) );
          mPanelScreen.PlayQuestion( lSide );
        }
      }
      break;
    }
  }

  /*
   * Return: true   play ready
   *         false  play not ready
   */
  public boolean                PlayExec()
  {
    boolean                     b = DoPlayExec();

    RefreshAnnoArea();

    return b;
  }

  private boolean               DoPlayExec()
  {
    boolean                     bReady = false;

    switch ( mSection )
    {
    case PbnGen.SECTION_IDENT:
      SetSection( PbnGen.SECTION_AUCTION );
      break;

    case PbnGen.SECTION_AUCTION:
      PbnAuction                lAuction = mGameData.GetAuction();
      if ( ! AuctionIsDone() )
      {
        char                lcIllegal;
        int                 iNote;

        PbnCall lCall = lAuction.GetCall( mCallIndex );
        switch ( lCall.GetType() )
        {
        case PbnCall.TYPE_INSUFF:
          lcIllegal = 'I';
          break;
        case PbnCall.TYPE_SKIP:
          lcIllegal = 'S';
          break;
        default:
          lcIllegal = '\0';
          break;
        }
        iNote = mGameTags.GetCallAnno( mCallIndex ).GetNote().Get();
        mPanelScreen.HistShow( GetRowForCall( mCallIndex )
                             , lCall, lcIllegal, iNote );
        mAnnoArea.ShowCall( mCallIndex );

        mCallIndex++;
        mPanelScreen.PlayClearSide( lCall.GetSide() );

        if ( ! AuctionIsDone() )
        {
          lCall = lAuction.GetCall( mCallIndex );
          mPanelScreen.HistQuestion( lCall.GetSide()
                                   , GetRowForCall( mCallIndex ) );
        }
      }
      else
      { /*
         * Auction completed.
         */
        SetSection( PbnGen.SECTION_PLAY );
      }
      break;

    case PbnGen.SECTION_PLAY:
      mPanelScreen.ClearOptNrTricks();
      if ( mbTrick )
      {
        mbTrick = false;

        int                     iIndex = mCardIndex - PbnSide.NUMBER;
        int                     iTrick = iIndex/PbnSide.NUMBER;
        PbnPlayCard             lPlayCard;
        PbnSide                 lSide;
        char                    lcIllegal;
        int                     iNote;

        for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
        {
          lPlayCard = mPlayAdmin.GetPlayCard( iIndex + iSide );
          lSide     = lPlayCard.GetSide();
          lcIllegal = mPlayAdmin.GetIllegal( iTrick, lSide );
          iNote     = mGameTags.GetCardAnno( iTrick, lSide ).GetNote().Get();
          mPanelScreen.HistShow( iIndex/PbnSide.NUMBER
                               , lPlayCard
                               , lcIllegal
                               , iNote );
        }
        mPanelScreen.PlayClearAll();

        SetNrTricks();

        if ( PlayIsDone() )
        {
          HandlePlayDone();
          bReady = true;
        }
        else
        {
          lPlayCard = mPlayAdmin.GetPlayCard( mCardIndex );
          mPanelScreen.PlayQuestion( lPlayCard.GetSide() );
          mPanelScreen.HistBar( lPlayCard.GetSide(), iTrick+1 );
        }
      }
      else
      if ( PlayIsDone() )
      {
        HandlePlayDone();
        bReady = true;
      }
      else
      {
        PbnPlayCard lPlayCard = mPlayAdmin.GetPlayCard( mCardIndex );
        PbnSide                 lSide = lPlayCard.GetSide();

        mDeal.PlayCard( lSide, lPlayCard.GetCard() );
        mPanelScreen.SetHand( lSide, mDeal.GetHand( lSide ) );

        char                    lcIllegal;
        lcIllegal = mPlayAdmin.GetIllegal( mCardIndex/PbnSide.NUMBER
                                         , lSide );
        mPanelScreen.PlayCard( lPlayCard
                             , lcIllegal
                             , 0 );
        mAnnoArea.ShowCard( mCardIndex );

        mCardIndex++;
        if ( (mCardIndex & 0x03) == 0 )
        {
          mbTrick = true;
        }
        else
        if ( ! PlayIsDone() )
        {
          lPlayCard = mPlayAdmin.GetPlayCard( mCardIndex );
          mPanelScreen.PlayQuestion( lPlayCard.GetSide() );
        }
      }
      break;
    }

    return bReady;
  }

  public boolean                PlayTail()
  {
    boolean                     bReady = false;

    switch ( mSection )
    {
    case PbnGen.SECTION_IDENT:
      break;

    case PbnGen.SECTION_AUCTION:
      while ( ! AuctionIsDone() )
      {
        DoPlayExec();
      }
      break;

    case PbnGen.SECTION_PLAY:
      while ( ! PlayIsDone() )
      {
        DoPlayExec();
      }
      if ( mbTrick )
      {
        DoPlayExec();
      }
      HandlePlayDone();
      bReady = true;
      break;
    }

    RefreshAnnoArea();
    return bReady;
  }

  public void                   ClearOptCards()
  {
    if ( ! ( PlayIsDone() ) )
    {
      PbnPlayCard lPlayCard = mPlayAdmin.GetPlayCard( mCardIndex );
      PbnSide lSide = lPlayCard.GetSide();

      mPanelScreen.SetHand( lSide
          , mDeal.GetHand( lSide ) );
      mPanelScreen.ClearOptNrTricks();
    }
  }

  public void                   ShowOptCards()
  {
    if ( mSection != PbnGen.SECTION_PLAY )
    {
      ClearOptCards();
      return;
    }

    if ( ! ( mbTrick || PlayIsDone() ) )
    {
      if ( maOptHands[ mCardIndex ] == null )
      {
        PbnGib          lGib = new PbnGib();

        if ( lGib != null )
        {
          maOptHands[ mCardIndex ] =
          lGib.GetOptCards( mDeal
                          , mGameData.GetContract().GetTrump()
                          , mPlayAdmin
                          , mCardIndex );
          maOptNrTricks[ mCardIndex ] = lGib.GetOptNrTricks();
          lGib.Quit();
        }
      }

      if ( maOptHands[ mCardIndex ] != null )
      {
        PbnPlayCard     lPlayCard = mPlayAdmin.GetPlayCard( mCardIndex );
        PbnSide         lSide = lPlayCard.GetSide();

        mPanelScreen.SetHand( lSide
                            , mDeal.GetHand( lSide )
                            , maOptHands[ mCardIndex ] );
        mPanelScreen.SetOptNrTricks( lSide
                                   , maOptNrTricks[ mCardIndex ] );
      }
    }
  }

  public void                   Rotate(
  int                           number )
  {
    mGameData.Rotate( number );
    mGameTags.Rotate( number );
  }
}
