/*
 * File   :     PbnMenu.java
 * Author :     Tis Veugen
 * Date   :     2012-04-18
 * PBN    :     2.1
 *
 * History
 * -------
 * 1999-08-13 Version 1.4
 * 1999-08-16 PBN versions
 *            wait cursor
 * 1999-08-25 Version 1.5
 * 1999-10-10 Version 2.0
 * 1999-11-24 Version 2.1
 * 2001-09-21 Updated cursor.
 *            Version 2.2
 * 2002-02-22 Added table header format
 *            Version 2.3
 * 2002-08-25 Check contract before playing
 *            Version 2.4
 * 2004-08-23 Added Rotate()
 * 2005-01-22 Adapted for GIB Version 6.1.3
 * 2007-08-27 Added VERSION_21
 * 2012-04-18 Catch exception
 *            Version 2.13
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class PbnMenu
implements ActionListener, ItemListener
{
  MenuBar                       mMenuBar;
  Menu                          mMenuProject,
                                mMenuGame,
                                mMenuAnal,
                                mMenuOptions;
  MenuItem                      mMiOpen,
                                mMiAbout,
                                mMiExit,
                                mMiFirst,
                                mMiNext,
                                mMiPrev,
                                mMiLast,
                                mMiRotate,
                                mMiAnno,
                                mMiFile,
                                mMiTable,
                                mMiDdOnce;
  RadioMenu                     mMenuVersion;
  CheckboxMenuItem              mMiPbn10,
                                mMiPbn20,
                                mMiPbn21,
                                mMiDdOn;
  Frame                         mFrame;
  PbnVwHndlr                    mPbnVwHndlr;
  String                        mDirectory = "";
  private RandomAccessFile      mImportRaf = null;

  public                        PbnMenu(
  Frame                         oFrame,
  PbnVwHndlr                    oPbnVwHndlr )
  {
    mFrame      = oFrame;
    mPbnVwHndlr = oPbnVwHndlr;

    mMenuBar = new MenuBar();
    mMenuProject = new Menu( "Project" );
    mMenuGame    = new Menu( "Game" );
    mMenuVersion = new RadioMenu( "PBN Version", true );
    mMenuAnal    = new Menu( "DD Analysis" );
    mMenuOptions = new Menu( "Options" );

    mMiOpen = addMenuItem( mMenuProject, "Open" );
    mMenuProject.addSeparator();
    mMiAbout = addMenuItem( mMenuProject, "About" );
    mMenuProject.addSeparator();
    mMiExit = addMenuItem( mMenuProject, "Exit" );
    mMenuBar.add( mMenuProject );

    mMiFirst = addMenuItem( mMenuGame, "First" );
    mMiNext  = addMenuItem( mMenuGame, "Next" );
    mMiPrev  = addMenuItem( mMenuGame, "Previous" );
    mMiLast  = addMenuItem( mMenuGame, "Last" );
    mMenuGame.addSeparator();
    mMiRotate = addMenuItem( mMenuGame, "Rotate" );
    mMenuBar.add( mMenuGame );

    mMiPbn10 = mMenuVersion.add( "PBN 1.0", this );
    mMiPbn20 = mMenuVersion.add( "PBN 2.0", this );
    mMiPbn21 = mMenuVersion.add( "PBN 2.1", this );
    mMiPbn21.setState( true );
    mMenuOptions.add( mMenuVersion );
    mMenuOptions.addSeparator();
    mMiAnno  = addMenuItem( mMenuOptions, "Annotation" );
    mMiFile  = addMenuItem( mMenuOptions, "PbnFile" );
    mMiTable = addMenuItem( mMenuOptions, "Tables" );
    mMenuOptions.addSeparator();
    mMiDdOnce = addMenuItem( mMenuAnal, "Once" );
    mMiDdOn   = addCbMenuItem( mMenuAnal, "On" );
    mMiDdOn.setState( false );
    mMenuOptions.add( mMenuAnal );
    mMenuBar.add( mMenuOptions );

    oFrame.setTitle( "PBN Viewer (PBN 1.0 + PBN 2.0 + PBN 2.1)" );
    oFrame.setMenuBar( mMenuBar );
  }

  private MenuItem              addMenuItem(
  Menu                          oMenu,
  String                        oString )
  {
    MenuItem                    lMenuItem = new MenuItem( oString );

    oMenu.add( lMenuItem );
    lMenuItem.addActionListener( this );

    return lMenuItem;
  }

  private CheckboxMenuItem      addCbMenuItem(
  Menu                          oMenu,
  String                        oString )
  {
    CheckboxMenuItem            lMenuItem = new CheckboxMenuItem( oString );

    oMenu.add( lMenuItem );
    lMenuItem.addItemListener( this );

    return lMenuItem;
  }

  public void                   actionPerformed(
  ActionEvent                   oEvent )
  {
    MenuItem                    lMenuItem = (MenuItem) oEvent.getSource();

    if ( lMenuItem == mMiOpen )
    {
      PbnFile                   lPbnFile = LoadPbnGames();

      if ( lPbnFile != null )
      {
        mPbnVwHndlr.NewGames( lPbnFile );
      }
    }
    else
    if ( lMenuItem == mMiAbout )
    {
      new PbnDialog( mFrame
                   , "Program Info"
                   , "Java PBN Viewer (v2.14) for PBN 2.1\n"
                     + "Made by Tis Veugen, 2012.04.29\n"
                     + "(http://www.tistis.nl/pbn)" );
    }
    else
    if ( lMenuItem == mMiExit )
    {
      mFrame.setVisible( false );
      System.exit( 0 );
      /***************/
    }
    else
    if ( lMenuItem == mMiFirst )
    {
      mPbnVwHndlr.GameFirst();
    }
    else
    if ( lMenuItem == mMiNext )
    {
      mPbnVwHndlr.GameNext();
    }
    else
    if ( lMenuItem == mMiPrev )
    {
      mPbnVwHndlr.GamePrev();
    }
    else
    if ( lMenuItem == mMiLast )
    {
      mPbnVwHndlr.GameLast();
    }
    else
    if ( lMenuItem == mMiRotate )
    {
      mPbnVwHndlr.Rotate();
    }
    else
    if ( lMenuItem == mMiAnno )
    {
      mPbnVwHndlr.Annotation();
    }
    else
    if ( lMenuItem == mMiFile )
    {
      mPbnVwHndlr.AnnoFile();
    }
    else
    if ( lMenuItem == mMiTable )
    {
      mPbnVwHndlr.AnnoTable();
    }
    else
    if ( lMenuItem == mMiDdOnce )
    {
      mPbnVwHndlr.AnalyseOnce( mFrame );
    }
  }

  public void                   itemStateChanged(
  ItemEvent                     oEvent )
  {
    MenuItem                    lMenuItem = (MenuItem) oEvent.getSource();

    if ( lMenuItem == mMiPbn10 )
    {
      PbnGen.SetVersion( PbnGen.VERSION_10 );
      mMenuVersion.SelectItem( lMenuItem );
    }
    else
    if ( lMenuItem == mMiPbn20 )
    {
      PbnGen.SetVersion( PbnGen.VERSION_20 );
      mMenuVersion.SelectItem( lMenuItem );
    }
    else
    if ( lMenuItem == mMiPbn21 )
    {
      PbnGen.SetVersion( PbnGen.VERSION_21 );
      mMenuVersion.SelectItem( lMenuItem );
    }
    else
    if ( lMenuItem == mMiDdOn )
    {
      if ( mMiDdOn.getState() )
      {
        mPbnVwHndlr.Analyse( mFrame );
      }
      else
      {
        mPbnVwHndlr.Analyse( null );
      }
    }
  }

  private PbnFile               LoadPbnGames()
  {
    String                      lFilename;

    lFilename = SetFile( "*.pbn"
                       , "Load PBN file"
                       , FileDialog.LOAD );
    if ( lFilename == null )
    {
      return null;
    }

    RandomAccessFile            lImportRaf;
    FileOutputStream            lLoggingFos;

    try
    {
      lImportRaf = new RandomAccessFile( lFilename, "r" );
    }
    catch ( Exception e )
    {
      new PbnDialog( mFrame
                   , "Error"
                   , "Can't open inputfile\n"
                     + "'" + lFilename + "'" );
      return null;
    }

    try
    {
      lLoggingFos = new FileOutputStream( "pbnJview.log" );
    }
    catch ( Exception e )
    {
      new PbnDialog( mFrame
                   , "Error"
                   , "Can't open logfile" );
      return null;
    }

    PbnInputStream              lPbnIs = new PbnInputStream( lImportRaf );
    PbnFile                     lPbnFile;

    lPbnFile = new PbnFile( lPbnIs, lLoggingFos );

    boolean bOk = true;
    mFrame.setCursor( new Cursor( Cursor.WAIT_CURSOR ) );
    //try
    {
      lPbnFile.Read();
    }/*
    catch ( Exception e )
    {
    	new PbnDialog( mFrame, "Program error"
                 , "You have found a bug !\n"
                 + "Please, send an e-mail with the PBN file to:\n"
                 + "tis.veugen@gmail.com" );
        bOk = false;
    }*/
    mFrame.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );

    try
    {
      lLoggingFos.flush();
      lLoggingFos.close();
    }
    catch ( Exception e )
    {
      // "Can't close log file"
    }

    if ( ! bOk )
    {
    	return null;
    }

    if ( mImportRaf != null )
    {
      try
      {
        mImportRaf.close();
      }
      catch ( Exception e )
      {
        // "Can't close PBN file"
      }
    }
    mImportRaf = lImportRaf;

    if ( lPbnFile.NrGames() == 0 )
    {
      new PbnDialog( mFrame
                   , "Error"
                   , "Inputfile '" + lFilename + "'\n"
                     + "has no (valid) PBN games" );
    }

    return lPbnFile;
  }

  private String                SetFile(
  String                        oFilter,
  String                        oHeader,
  int                           iMode )
  {
    FileDialog                  lFd;
    String                      lFilename;

    lFd = new FileDialog( mFrame
                        , oHeader
                        , iMode );
    lFd.setDirectory( mDirectory );
    lFd.setFile( oFilter );
    lFd.show();

    lFilename = lFd.getFile();
    if ( lFilename != null )
    {
      mDirectory = lFd.getDirectory();
      lFilename = mDirectory + lFilename;
    }

    return lFilename;
  }
}

class RadioMenu extends Menu
{
  public RadioMenu( String s, boolean t )
  {
    super( s, t );
  }

  public CheckboxMenuItem       add(
  String                        oString,
  ItemListener                  oListener )
  {
    CheckboxMenuItem            lMenuItem = new CheckboxMenuItem( oString );

    add( lMenuItem );
    lMenuItem.addItemListener( oListener );

    return lMenuItem;
  }

  public void SelectItem( MenuItem oItem )
  {
    CheckboxMenuItem  lItem;
    int               nItems = getItemCount();

    for ( int i=0; i<nItems; i++ )
    {
      if ( oItem != getItem(i) )
      {
        lItem = (CheckboxMenuItem) getItem(i);
        lItem.setState( false );
      }
    }
  }
}
