/*
 * File   :     PbnPlayAdmin.java
 * Author :     Tis Veugen
 * Date   :     2003-03-29
 * PBN    :     2.0
 *
 * History
 * -------
 * 2003-03-29 Take unknown cards into account
 */

public class PbnPlayAdmin
{
  private final int             NR_PLAY_CARDS = PbnTrick.NUMBER * PbnSide.NUMBER;

  private int                   mNrPlayCards;
  private PbnPlayCard []        maPlayCards;
  private char [][]             maacIllegals;
  private int []                maNrWonNS;
  private int []                maNrWonEW;

  public			PbnPlayAdmin()
  {
    mNrPlayCards = 0;
    maPlayCards = new PbnPlayCard[ NR_PLAY_CARDS ];
    maacIllegals = new char[ PbnTrick.NUMBER ][ PbnSide.NUMBER ];
    maNrWonNS = new int[ PbnTrick.NUMBER ];
    maNrWonEW = new int[ PbnTrick.NUMBER ];

    for ( int i = 0; i < NR_PLAY_CARDS; i++ )
    {
      maPlayCards[ i ] = new PbnPlayCard();
    }
  }

  public PbnPlayCard            GetPlayCard(
  int                           iIndex )
  {
    return maPlayCards[ iIndex ];
  }

  public char                   GetIllegal(
  int                           iTrick,
  PbnSide                       oSide )
  {
    return maacIllegals[ iTrick ][ oSide.Get() ];
  }

  public int                    GetNrWonNS(
  int                           iTrick )
  {
    return maNrWonNS[ iTrick ];
  }

  public int                    GetNrWonEW(
  int                           iTrick )
  {
    return maNrWonEW[ iTrick ];
  }

  private void                  PrepareCard(
  PbnDeal                       oDeal,
  PbnSide                       oSide,
  PbnCard                       oCard )
  {
    oDeal.PlayCard( oSide, oCard );
  }

  /*
   * Partly copied from PbnExport.
   */
  public void                   Compute(
  PbnGameData                   oGameData )
  {
    PbnSide                     lSide = new PbnSide();
    PbnSide                     lPlaySide;
    PbnPlay                     lPlay;
    PbnDeal                     lDeal;
    boolean []                  labUnknownSides;
    int                         NrWonNS = 0;
    int                         NrWonEW = 0;
    PbnCard                     lLeadCard;
    PbnCard                     lPlayCard;
    PbnSide                     lWinnerSide = new PbnSide();
    PbnCard                     lWinCard = new PbnCard();
    PbnSuit                     lTrumpSuit;

    lPlay = oGameData.GetPlay();
    lDeal = new PbnDeal( oGameData.GetDeal() );

    labUnknownSides = new boolean[ PbnSide.NUMBER ];
    /*
     * Hands with no cards are considered as unknown hands.
     */
    lSide.Set( PbnSide.SOUTH );
    for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
    {
      labUnknownSides[ iSide ] = ( lDeal.GetNrRanks( lSide ) == 0 );
      lSide.Next();

      for ( int iTrick = 0; iTrick < PbnTrick.NUMBER; iTrick++ )
      {
        maacIllegals[ iTrick ][ iSide ] = '\0';
      }
    }

    lPlaySide = new PbnSide( oGameData.GetContract().GetDeclarer() );
    lPlaySide.Next();

    for ( int iTrick = 0; iTrick < PbnTrick.NUMBER; iTrick++ )
    { /*
       * Check if any cards are played.
       */
      int                       NrPlayed = 0;
      boolean                   bUnknown = false; /* no Unknown cards yet */

      /*
       * Already copy, to cope with unknown cards.
       */
      maNrWonNS[ iTrick ] = NrWonNS;
      maNrWonEW[ iTrick ] = NrWonEW;

      lSide.Set( PbnSide.SOUTH );
      for ( int iSide = 0; iSide < PbnSide.NUMBER; iSide++ )
      {
        if ( lPlay.GetCard( iTrick, lSide ).IsUsed() )
        {
          NrPlayed++;
        }
        lSide.Next();
      }

      if ( NrPlayed == 0 )
      {
        break;
      }

      PbnSide lFirstSide = lPlay.GetFirst( iTrick );
      if ( lFirstSide.IsValid() )
      {
        lPlaySide.Set( lFirstSide );
        maacIllegals[ iTrick ][ lPlaySide.Get() ] = 'L';
      }

      lLeadCard = lPlay.GetCard( iTrick, lPlaySide );

      maPlayCards[ mNrPlayCards ].GetCard().Set( lLeadCard );
      maPlayCards[ mNrPlayCards ].GetSide().Set( lPlaySide );
      mNrPlayCards++;

      if ( ( lLeadCard.GetSuit().Is( PbnSuit.UNKNOWN ) )
        || ( lLeadCard.GetRank().IsNone()              ) )
      { /*
         * When the leader doesn't play, then a revoke can't be tested.
         */
        continue;
        /*******/
      }

      PrepareCard( lDeal, lPlaySide, lLeadCard );

      /*
       * Determine the winner side.
       */
      lWinnerSide.Set( lPlaySide );
      lWinCard.Set( lLeadCard );
      lTrumpSuit = new PbnSuit( oGameData.GetContract().GetTrump() );

      for ( int iSide = 1; iSide < PbnSide.NUMBER; iSide++ )
      {
        lPlaySide.Next();
        lPlayCard = lPlay.GetCard( iTrick, lPlaySide );

        maPlayCards[ mNrPlayCards ].GetCard().Set( lPlayCard );
        maPlayCards[ mNrPlayCards ].GetSide().Set( lPlaySide );
        mNrPlayCards++;

        if ( ( lPlayCard.GetSuit().Is( PbnSuit.UNKNOWN ) )
          || ( lPlayCard.GetRank().IsNone()              ) )
        {
          bUnknown = true;

          continue;
          /*******/
        }

        if ( ! lLeadCard.GetSuit().equals( lPlayCard.GetSuit() ) )
        {
          if ( ( ! labUnknownSides[ lPlaySide.Get() ] )
            && ( lDeal.GetNrRanks( lPlaySide, lLeadCard.GetSuit() ) > 0 ) )
          { /*
             * Wrong suit played.
             */
            maacIllegals[ iTrick ][ lPlaySide.Get() ] = 'R';
          }
        }

        if ( ( lPlayCard.GetSuit().equals( lTrumpSuit )  )
          && ( ! lWinCard.GetSuit().equals( lTrumpSuit ) ) )
        { /*
           * A trump card has been played for the first time.
           */
          lWinCard.GetSuit().Set( lTrumpSuit );
          lWinCard.GetRank().Set( PbnRank.NONE );
        }

        if ( ( lWinCard.GetSuit().equals( lPlayCard.GetSuit() ) )
          && ( lWinCard.GetRank().LT( lPlayCard.GetRank() )     ) )
        {
          lWinCard.GetRank().Set( lPlayCard.GetRank() );
          lWinnerSide.Set( lPlaySide );
        }

        PrepareCard( lDeal, lPlaySide, lPlayCard );
      }

      lPlaySide.Set( lWinnerSide );
      if ( ! bUnknown )
      {
        if ( lWinnerSide.IsNS() )
        {
          NrWonNS++;
        }
        else
        {
          NrWonEW++;
        }
      }
      maNrWonNS[ iTrick ] = NrWonNS;
      maNrWonEW[ iTrick ] = NrWonEW;
    }

    if ( mNrPlayCards < NR_PLAY_CARDS )
    {
      maPlayCards[ mNrPlayCards ].GetSide().Set( PbnSide.NONE );
    }
  }

  public boolean                PlayIsDone(
  int                           iIndex )
  {
    return iIndex >= mNrPlayCards;
  }
}
