/*
 * File   :     PbnVwHndlr.java
 * Author :     Tis Veugen
 * Date   :     2004-08-23
 * PBN    :     2.0
 *
 * History
 * -------
 * 2002-02-22 Changed mAnnotation
 * 2004-08-23 Added Rotate()
 */

import java.awt.*;
import java.awt.Cursor;

public class PbnVwHndlr
  implements TimerListener
{
  private PanelScreen           mPanelScreen;
  private int                   mGameIndex = 0;
  private int                   mGameMax = 0;
  private int                   mRotate = 0;
  private PbnFile               mPbnFile;
  private PbnGame               mCurrentGame;
  private PbnTimer              mTimer = null;
  private int                   mTimerId = 0;
  private int                   mAnnotation = PanelScreen.ANNOTATION;

  private boolean               mAnalyse = false;
  private Frame                 mFrame = null;

  public                        PbnVwHndlr()
  {
  }

  public void                   SetPanelScreen(
  PanelScreen                   oPanelScreen )
  {
    mPanelScreen = oPanelScreen;
  }

  public void                   SetSection(
  int                           iSection )
  {
    if ( mGameMax == 0 ) return;

    mCurrentGame.SetSection( iSection );
    AnalyseExec();
  }

  public void                   SetHidden(
  boolean []                    abHiddenSides )
  {
    if ( mGameMax == 0 ) return;

    mCurrentGame.SetHidden( abHiddenSides );
  }

  public void                   SetGameIndex(
  int                           iGameIndex )
  {
    mGameIndex = iGameIndex;
    mRotate = 0;
    SetGame();
  }

  private void                   SetGame()
  {
    mPanelScreen.SetGameNr( mGameIndex );
    mPanelScreen.ClearAnno();

    mCurrentGame = mPbnFile.GetGame( mGameIndex );
    if ( mRotate != 0 )
    {
      mCurrentGame.Rotate( mRotate );
    }
    mCurrentGame.New( mPanelScreen );

    mPanelScreen.SetAnno( mAnnotation );
    mPanelScreen.Rotate( mRotate );
  }

  public void                   NewGames(
  PbnFile                       oPbnFile )
  {
    mPbnFile = oPbnFile;
    mGameMax = mPbnFile.NrGames();

    if ( mGameMax == 0 )
    {
      mPanelScreen.Reset();
    }
    else
    {
      mPanelScreen.SetGameNr( 0 );
      mPanelScreen.SetGameMax( mGameMax );
      GameFirst();
    }
  }

  public void                   GameFirst()
  {
    if ( mGameMax == 0 ) return;

    SetGameIndex( 0 );
  }

  public void                   GameNext()
  {
    if ( mGameMax == 0 ) return;

    if ( mGameIndex+1 < mGameMax )
    {
      mGameIndex++;
    }
    SetGameIndex( mGameIndex );
  }

  public void                   GamePrev()
  {
    if ( mGameMax == 0 ) return;

    if ( mGameIndex > 0 )
    {
      mGameIndex--;
    }
    SetGameIndex( mGameIndex );
  }

  public void                   GameLast()
  {
    if ( mGameMax == 0 ) return;

    SetGameIndex( mGameMax-1 );
  }

  public void                   Rotate()
  {
    if ( mGameMax == 0 ) return;

    mRotate++;
    if ( mRotate == PbnSide.NUMBER ) mRotate = 0;
    SetGame();
  }

  public void                   Annotation()
  {
    mAnnotation = PanelScreen.ANNOTATION;
    if ( mGameMax == 0 ) return;

    mPanelScreen.SetAnno( mAnnotation );
  }

  public void                   AnnoFile()
  {
    mAnnotation = PanelScreen.ANNO_FILE;
    if ( mGameMax == 0 ) return;

    mPanelScreen.SetAnno( mAnnotation );
  }

  public void                   AnnoTable()
  {
    mAnnotation = PanelScreen.ANNO_TABLE;
    if ( mGameMax == 0 ) return;

    mPanelScreen.SetAnno( mAnnotation );
  }

  public void                   PlayHead()
  {
    if ( mGameMax == 0 ) return;

    mCurrentGame.PlayHead();
    AnalyseExec();
  }

  public void                   PlayBack()
  {
    if ( mGameMax == 0 ) return;

    mCurrentGame.PlayBack();
    AnalyseExec();
  }

  public void                   PlayStop()
  {
    if ( mGameMax == 0 ) return;

    if ( mTimer != null )
    {
      mTimer.Stop();
      mTimer = null;
      mTimerId = 0;
    }
  }

  public void                   PlayExec()
  {
    if ( mGameMax == 0 ) return;

    if ( mCurrentGame.PlayExec() )
    {
      PlayStop();
    }
    AnalyseExec();
  }

  public void                   PlayAuto()
  {
    if ( mGameMax == 0 ) return;

    if ( mTimer == null )
    {
      mTimer = new PbnTimer();
      mTimer.addTimerListener( this );
      mTimerId = mTimer.Start( 1000 );
    }
  }

  public void                   PlayTail()
  {
    if ( mGameMax == 0 ) return;

    if ( mCurrentGame.PlayTail() )
    {
      PlayStop();
    }
  }

  public void                   timeElapsed(
  TimerEvent                    oTimerEvent )
  {
    if ( oTimerEvent.mId == mTimerId )
    {
      PlayExec();
    }
  }

  private void CursorWait()
  {
    mFrame.setCursor( new Cursor( Cursor.WAIT_CURSOR ) );
  }

  private void CursorDefault()
  {
    mFrame.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
  }

  private void AnalyseExec()
  {
    if ( mAnalyse )
    {
      AnalyseGame();
    }
  }

  private void AnalyseGame()
  {
    CursorWait();
    mCurrentGame.ShowOptCards();
    CursorDefault();
  }

  public void Analyse( Frame oFrame )
  {
    mFrame = oFrame;
    mAnalyse = ( mFrame != null );
    if ( mAnalyse )
    {
      AnalyseExec();
    }
    else
    {
      mCurrentGame.ClearOptCards();
    }
  }

  public void AnalyseOnce( Frame oFrame )
  {
    if ( mGameMax == 0 ) return;

    mFrame = oFrame;
    AnalyseGame();
  }
}
